#!/usr/bin/env python

# Copyright (c) 2009, Hoyt Koepke (hoytak@gmail.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     - Neither the name 'treedict' nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Hoyt Koepke ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Hoyt Koepke BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import division

import random, unittest, cPickle, collections
from treedict import TreeDict, getTree
import treedict
from copy import deepcopy, copy
from cPickle import dumps
from hashlib import md5
import random

from common import *

class TestConstraints(unittest.TestCase):
    def setUp(self):
        treedict.addConstraint("a_is_int", "a", lambda x: type(x) is int, "Not an integer.")
        treedict.addConstraint("a_in_125", "a", lambda x: x == 1 or x == 2 or x == 5, "Not 1,2,or 5.")
        treedict.addGlobalConstraint("in_134", set([1,3,4]), "Not 1, 3, or 4")
        treedict.addGlobalConstraint("l_in_134", [1,3,4], "Not 1, 3, or 4")
        treedict.addGlobalConstraint("in_134l", [1,3,4,[]], "Not 1, 3, 5 or []")
       

    # treedict.addConstraint("a_is_int", "a", lambda x: type(x) is int, "Not an integer.")
    def testConstraint_01_pos(self):
        p1 = getTree("a_is_int")
        p1.a = 15

    def testConstraint_01_neg1(self):
        def f():
            p1 = getTree("a_is_int")
            p1.a = 15.3

        self.assertRaises(ValueError, f)

    def testConstraint_01_neg2(self):
        def f():
            p1 = getTree("a_is_int")
            p1.a.b = 0

        self.assertRaises(ValueError, f)

    def testConstraint_01_only_for_getTree(self):
        p1 = TreeDict("a_is_int")
        p1.a = 15.3

    # treedict.addConstraint("a_in_125", "a", lambda x: x == 1 or x == 2 or x == 5, "Not 1,2,or 5.")
    def testConstraint_02_pos(self):
        p1 = getTree("a_in_125")
        p1.a = 5

    def testConstraint_02_neg1(self):
        def f():
            p1 = getTree("a_in_125")
            p1.a = 4

        self.assertRaises(ValueError, f)

    def testConstraint_02_neg2(self):
        def f():
            p1 = getTree("a_in_125")
            p1.a.b = 0

        self.assertRaises(ValueError, f)

    def testConstraint_02_neg3(self):
        def f():
            p1 = getTree("a_in_125")
            p1.a = None

        self.assertRaises(ValueError, f)

    # treedict.addGlobalConstraint("in_134", set(1,3,4), "Not 1, 3, or 4")
    def testConstraint_03_pos(self):
        p1 = getTree(unique_name())
        p1.in_134 = 4

    def testConstraint_03_neg1(self):
        def f():
            p1 = getTree(unique_name())
            p1.in_134 = 5

        self.assertRaises(ValueError, f)

    def testConstraint_03_neg2(self):
        def f():
            p1 = getTree(unique_name())
            p1.in_134.b = 123

        self.assertRaises(ValueError, f)

    #treedict.addGlobalConstraint("l_in_134", list(1,3,4), "Not 1, 3, or 4")
    def testConstraint_04_pos(self):
        p1 = getTree(unique_name())
        p1.l_in_134 = 4

    def testConstraint_04_neg1(self):
        def f():
            p1 = getTree(unique_name())
            p1.l_in_134 = 5

        self.assertRaises(ValueError, f)

    def testConstraint_04_neg2(self):
        def f():
            p1 = getTree(unique_name())
            p1.l_in_134.b = 123

        self.assertRaises(ValueError, f)

    # treedict.addGlobalConstraint("in_134l", list(1,3,4,[]), "Not 1, 3, 5 or []")
    def testConstraint_05_pos(self):
        p1 = getTree(unique_name())
        p1.in_134l = []

    def testConstraint_05_neg1(self):
        def f():
            p1 = getTree(unique_name())
            p1.in_134l = 5

        self.assertRaises(ValueError, f)

    def testConstraint_05_neg2(self):
        def f():
            p1 = getTree(unique_name())
            p1.in_134l.b = []

        self.assertRaises(ValueError, f)


    def testConstraint_06_afterwards_1(self):
        tn = unique_name()
        bn = unique_name()

        p = getTree(tn)
        p[bn] = "borkbork"
        
        def f():
            treedict.addConstraint(tn, bn,["bork", "bork1"])

        self.assertRaises(ValueError, f)

    def testConstraint_06_afterwards_2(self):
        tn = unique_name()
        bn = unique_name()

        p = getTree(tn)
        p[bn] = "bork"  # passes
        
        treedict.addConstraint(unique_name(), bn,["bork", "bork1"])
        

    def testConstraint_07_afterwards_global_1(self):
        tn = unique_name()
        bn = unique_name()

        p = getTree(tn)
        p[bn] = "borkbork"
        
        def f():
            treedict.addGlobalConstraint(bn, ["bork", "bork1"])

        self.assertRaises(ValueError, f)

    def testConstraint_07_afterwards_global_2(self):
        tn = unique_name()
        bn = unique_name()

        p = getTree(tn)
        p[bn] = "bork"
        
        treedict.addGlobalConstraint(bn, ["bork", "bork1"])

    def testConstraint_06_afterwards_1_copy(self):
        tn = unique_name()
        bn = unique_name()

        p = getTree(tn)
        
        p[bn] = "bork"     # passes 
        q = p.copy(deep = True)
        q[bn] = "borkbork" # causes failure
        
        def f():
            treedict.addConstraint(tn, bn, ["bork", "bork1"])

        self.assertRaises(ValueError, f)

    def testConstraint_06_afterwards_2_copy(self):
        tn = unique_name()
        bn = unique_name()

        p = getTree(tn)
        p[bn] = "bork"  # passes
        q = p.copy(deep = True)
        q[bn] = "bork1"  # passes

        treedict.addConstraint(tn, bn, ["bork", "bork1"])
        
    def testConstraint_06_afterwards_3_copy_large(self):
        tn = unique_name()
        bn = unique_name()

        p = getTree(tn)

        for i in range(100):
            p[bn] = "bork"  # passes
            p = p.copy()

        p[bn] = "borkbork"  # fails

        def f():
            treedict.addConstraint(tn, bn, ["bork", "bork1"])

        self.assertRaises(ValueError, f)

    def testConstraint_07_afterwards_global_1_copy(self):
        tn = unique_name()
        bn = unique_name()

        p = getTree(tn)
        b = p.copy()
        b[bn]= 2
        
        def f():
            treedict.addGlobalConstraint(bn, [1], "not 1")

        self.assertRaises(ValueError, f)

    def testConstraint_07_afterwards_global_1_copy_control(self):
        tn = unique_name()
        bn = unique_name()

        p = getTree(tn)
        b = p.copy()
        b[bn] = 1
        
        # Should be fine
        treedict.addGlobalConstraint(bn, [1], "not 1")
        

    def testConstraint_08_branch_copy_1(self):
        tn = unique_name()
        bn = unique_name()

        treedict.addConstraint(tn, 'a.b.c', [1], "not 1")
        
        p = getTree(tn)
        p.a.makeBranch('b')
        q = p.a.b.copy()

        def f():
            q.c = 2  # should be bad

        self.assertRaises(ValueError, f)

    def testConstraint_08_branch_copy_2(self):
        tn = unique_name()

        treedict.addConstraint(tn, 'a.b.c', [1], "not 1")
        
        p = getTree(tn)
        p.makeBranch('a.b')
        q = p.a.b.copy()

        def f():
            q.c = 2  # should be bad

        self.assertRaises(ValueError, f)


    def testConstraint_08_branch_copy_afterwards_1(self):
        tn = unique_name()

        p = getTree(tn)
        p.makeBranch('a18462.a1732643')
        q = p.a18462.copy()
        q.a1732643.a1232 = "borkbork"  # should be bad

        def f():
            treedict.addConstraint(tn, 'a18462.a1732643.a1232', ["bork"])
        
            
        self.assertRaises(ValueError, f)

    def testConstraint_08_branch_copy_afterwards_2(self):
        tn = unique_name()

        p = getTree(tn)
        p.makeBranch('a18462.a1732643')

        q = p.a18462.a1732643.copy()
        q.a1232 = "borkbork"  # should be bad

        def f():
            treedict.addConstraint(tn, 'a18462.a1732643.a1232', ["bork"])

        self.assertRaises(ValueError, f)


    def testConstraint_08_global_branch_copy_afterwards_1(self):
        bn = unique_name()

        p = getTree(unique_name())
        p.makeBranch(bn).makeBranch('a')

        q = p[bn].copy()
        q.a.b = "borkbork"  # should be bad

        def f():
            treedict.addGlobalConstraint(bn + '.a.b', ["bork"])
            
        self.assertRaises(ValueError, f)

    def testConstraint_08_global_branch_copy_afterwards_1_control(self):
        bn = unique_name()

        p = getTree(unique_name())
        p.makeBranch(bn).makeBranch('a')

        q = p[bn].copy()
        q.a.b = "bork"  # should be bad

        treedict.addGlobalConstraint(bn + '.a.b', ["bork"])

    # Test for tree names that look like branch names
    def testConstraint_09_branches_in_tree_names_01(self):
        tn = unique_name()
        tnb = tn + '.b'

        p = getTree(tnb)
        treedict.addConstraint(tn, 'b.a', [1], "not 1")

        # shouldn't throw
        p.a = 2
        
    # Test for tree names that look like branch names
    def testConstraint_09_branches_in_tree_names_02(self):
        tn = unique_name()

        p = getTree(tn)

        treedict.addConstraint(tn + '.b', 'a', [1], "not 1")

        # shouldn't throw
        p.b.a = 2
        
    # Test for tree names that look like branch names
    def testConstraint_09_branches_in_tree_names_03(self):
        tn = unique_name()
        tnb = tn + '.b'

        p = getTree(tnb)

        treedict.addConstraint(tnb, 'a', [1], "not 1")

        # should throw
        def f():
            p.a = 2

        self.assertRaises(ValueError, f)

    # Test for tree names that look like branch names
    def testConstraint_09_branches_in_tree_names_03_control(self):
        tn = unique_name()
        tnb = tn + '.b'
        p = getTree(tnb)

        treedict.addConstraint(tnb, 'a', [1], "not 1")

        # shouldn't throw
        p.a = 1

    def testConstraint_10_copying_01(self):

        tn, bn = unique_name(), unique_name()

        p1 = getTree(tn)
        p1.makeBranch(bn)

        p2 = getTree(bn)
        p1c = p1[bn].copy()

        treedict.addConstraint(tn, bn +'.a', [1], "not 1")
        
        # Should work
        p2.a = 2

        def f():
            # Should throw, cause it's part of p1's family
            p1c.a = 2

        self.assertRaises(ValueError, f)

        # Shouldn't throw, cause it passes
        p1c.a = 1

        
    def testConstraint_10_copying_02(self):
        tn, bn = unique_name(), unique_name()

        p1 = getTree(tn)
        p1c = p1.makeBranch(bn).copy()
        p2 = getTree(bn)
        
        treedict.addConstraint(bn, 'a', [1], "not 1")

        # Should work fine; not part of p2's family
        p1c.a = 2
        self.assert_(p1c.branchName() == bn)
        
        # Should throw, cause it's part of p2's family
        def f(): p2.a = 2

        self.assertRaises(ValueError, f)

        # Shouldn't throw, it passes
        p2.a = 1


    def testConstraint_11_copying_01_afterwards(self):

        tn, bn = unique_name(), unique_name()

        p1 = getTree(tn)
        p1.makeBranch(bn)

        p2 = getTree(bn)

        p1c = p1[bn].copy()

        treedict.addConstraint(tn, bn +'.a', [1], "not 1")
        
        # Should work
        p2.a = 2
        
        def f():
            # Should throw, cause it's part of p1's family
            p1c.a = 2

        self.assertRaises(ValueError, f)

        # Shouldn't throw, cause it passes
        p1c.a = 1

        
    def testConstraint_11_copying_02_aferwards(self):
        tn, bn = unique_name(), unique_name()

        p1 = getTree(tn)
        p1c = p1.makeBranch(bn).copy()
        p2 = getTree(bn)
        
        
        # Should work fine; not part of p2's family
        p1c.a = 2
        self.assert_(p1c.branchName() == bn)
        
        # Should cause throw, cause it's part of p2's family
        p2.a = 2
        
        def f(): treedict.addConstraint(bn, 'a', [1], "not 1")

        self.assertRaises(ValueError, f)

    def testConstraint_15_no_change_on_failure(self):
        tn = unique_name()
        
        p = getTree(tn)
        p.b   = 123
        p.c.b = 'bork'
        
        p1 = p.copy()

        treedict.addConstraint(tn, 'a.b.c.d', [1], "not 1")

        def f():
            p["a.b.c.d"] = 2
            
        self.assertRaises(ValueError, f)

        self.assert_(p == p1)
        
    def testConstraint_16_constraint_not_added_on_failure(self):
        tn = unique_name()
        
        p = getTree(tn)
        p["a.b.c.d"] = 2

        def f():
            treedict.addConstraint(tn, 'a.b.c.d', [1], "not 1")
            
        self.assertRaises(ValueError, f)

        # Should be fine still, as 
        p["a.b.c.d"] = 2

    def testConstraint_16_constraint_not_added_on_failure__control(self):
        tn = unique_name()
        
        p = getTree(tn)
        p["a.b.c.d"] = 1

        treedict.addConstraint(tn, 'a.b.c.d', [1], "not 1")
            
        # Should now throw, as the constraint is in place
        def f():
            p["a.b.c.d"] = 2

        self.assertRaises(ValueError, f)


    def testConstraint_17_global_constraint_not_added_on_failure(self):
        tn1 = unique_name()
        tn2 = unique_name()
        bn = unique_name()
        
        p1 = getTree(tn1)
        p1[bn] = 2

        p2 = getTree(tn2)

        def f():
            treedict.addGlobalConstraint(bn, [1], "not 1")
            
        self.assertRaises(ValueError, f)

        # Should be fine still, as the constraint should be backed out
        p2[bn] = 2


    def testConstraint_17_constraint_not_added_on_failure__control(self):
        tn1 = unique_name()
        tn2 = unique_name()
        bn = unique_name()
        
        p1 = getTree(tn1)
        p1[bn] = 1

        p2 = getTree(tn2)

        treedict.addGlobalConstraint(bn, [1], "not 1")
            
        # Should throw, as the constraint succeeded
        def f():
            p2[bn] = 2

        self.assertRaises(ValueError, f)

        # Control, should pass
        p2[bn] = 1


    def testConstraint_18_copying_original_tracks_constraints_01(self):

        tn = unique_name()
        bn = unique_name()
        
        p1 = getTree(tn)
        p2 = p1.copy()

        treedict.addConstraint(tn, bn, [1], "not 1")
            
        # Should throw
        def f():
            p2[bn] = 2

        self.assertRaises(ValueError, f)

        # Control, should pass
        p2[bn] = 1

    def testConstraint_18_copying_original_tracks_constraints_02(self):

        tn = unique_name()
        bn = unique_name()
        
        p = getTree(tn)
        p.a = 1
        p.b = 2

        for i in range(1000):
            p = p.copy(deep = True)

        treedict.addConstraint(tn, bn, [1], "not 1")
            
        # Should throw
        def f():
            p[bn] = 2

        self.assertRaises(ValueError, f)

        # control, should pass
        p[bn] = 1


    def testConstraint_18_copying_original_tracks_constraints_01_after(self):

        tn = unique_name()
        bn = unique_name()
        
        p1 = getTree(tn)
        p2 = p1.copy()

        p2[bn] = 2

        # Should throw
        def f():
            treedict.addConstraint(tn, bn, [1], "not 1")

        self.assertRaises(ValueError, f)

        # The control; should pass
        treedict.addConstraint(tn, bn, [2], "not 2")

    def testConstraint_18_copying_original_tracks_constraints_02_after(self):

        tn = unique_name()
        bn = unique_name()
        
        p = getTree(tn)
        p.a = 1
        p.b = 2

        for i in range(1000):
            p = p.copy(deep = True)

        p[bn] = 2
            
        # Should throw
        def f():
            treedict.addConstraint(tn, bn, [1], "not 1")

        self.assertRaises(ValueError, f)

        # The control; should pass
        treedict.addConstraint(tn, bn, [2], "not 2")

    def testConstraint_19_global__copying_original_tracks_constraints_01(self):

        tn = unique_name()
        bn = unique_name()
        
        p1 = getTree(tn)
        p2 = p1.copy()

        treedict.addGlobalConstraint(bn, [1], "not 1")

        # Should throw
        def f():
            p2[bn] = 2

        self.assertRaises(ValueError, f)

        # The control
        p2[bn] = 1

    def testConstraint_19_global__copying_original_tracks_constraints_02(self):

        tn = unique_name()
        bn = unique_name()
        
        p = getTree(tn)
        p.a = 1
        p.b = 2

        for i in range(1000):
            p = p.copy(deep = True)

        treedict.addGlobalConstraint(bn, [1], "not 1")
            
        # Should throw
        def f():
            p[bn] = 2

        self.assertRaises(ValueError, f)

        # The control
        p[bn] = 1

    def testConstraint_19_global__copying_original_tracks_constraints_01_after(self):

        tn = unique_name()
        bn = unique_name()
        
        p1 = getTree(tn)
        p2 = p1.copy()

        p2[bn] = 2

        # Should throw
        def f():
            treedict.addGlobalConstraint(bn, [1], "not 1")

        self.assertRaises(ValueError, f)

        # And the control
        treedict.addGlobalConstraint(bn, [2], "not 2")


    def testConstraint_19_global__copying_original_tracks_constraints_02_after(self):

        tn = unique_name()
        bn = unique_name()
        
        p = getTree(tn)
        p.a = 1
        p.b = 2

        for i in range(1000):
            p = p.copy(deep = True)

        p[bn] = 2
            
        # Should throw
        def f():
            treedict.addGlobalConstraint(bn, [1], "not 1")

        self.assertRaises(ValueError, f)

        # And the control
        treedict.addGlobalConstraint(bn, [2], "not 2")

    def testConstraint_20_global__branches_track_original__after(self):
        
        tn = unique_name()
        bn1 = unique_name()
        bn2 = unique_name()

        p = getTree(tn)

        p.makeBranch(bn1 + '.' + bn2)
        b1 = p[bn1].copy()
        b2 = b1[bn2].copy()
        
        b2.x = 1
        b1[bn2]['x'] = 2

        def f():
            treedict.addGlobalConstraint('%s.%s.x' % (bn1, bn2), [1], "not 1")

            
        self.assertRaises(ValueError, f)


    def testConstraint_21_global__branches_track_original_deep(self):
        n = 100
        tn = unique_name()
        bnl = [unique_name() for i in range(n)]
        p = getTree(tn)

        fullbn = '.'.join(bnl)

        p.makeBranch(fullbn)

        bl = [None]*n

        for i, bn in enumerate(bnl):
            bl[i] = (p if i == 0 else bl[i-1])[bn]

        treedict.addGlobalConstraint(fullbn + '.x', [1], "not 1")

        for i, b in enumerate(bnl[:-1]):

            def f():
                bl[i]['.'.join(bnl[i+1:]) + '.x'] = 2

            self.assertRaises(ValueError, f)

        for i, b in enumerate(bnl[:-1]):
            bl[i]['.'.join(bnl[i+1:]) + '.x'] = 1


    def testConstraint_21_global__branches_track_original_deep__after(self):
        n = 10
        tn = unique_name()
        bnl = [unique_name() for i in range(n)]
        p = getTree(tn)

        fullbn = '.'.join(bnl)

        p.makeBranch(fullbn)

        bl = [None]*n

        bcopy = [None]*n

        for i, bn in enumerate(bnl):
            bl[i] = (p if i == 0 else bl[i-1])[bn]

        def get_x_name(i):
            return '.'.join(bnl[i+1:] + ['x'])

        self.assert_(get_x_name(n-1) == 'x')
        self.assert_(get_x_name(n-2) == '%s.x' % bnl[-1])
        self.assert_(get_x_name(0) == '%s.x' % '.'.join(bnl[1:]))

        for i, b in enumerate(bnl):
            bcopy[i] = bl[i].copy()
            bcopy[i][get_x_name(i)] = 1

        n2 = n // 2

        bcopy[n2][get_x_name(n2)] = 2

        def f():
            treedict.addGlobalConstraint(fullbn + '.x', [1], "not 1")
            
        self.assertRaises(ValueError, f)
        

    def testConstraint_22_IntermediatesMustBeBranches_01(self):
        tn = unique_name()
        
        treedict.addConstraint(tn, 'a.b.c.d.v', [1], "not 1")

        p = treedict.getTree(tn)

        def f():
            p["a.b"] = 1

        self.assertRaises(ValueError, f)

        # control
        p.a.makeBranch("b")

    def testConstraint_22_IntermediatesMustBeBranches_02_after(self):
        tn = unique_name()
        
        p = treedict.getTree(tn)

        p["a.b"] = 1

        def f():
            treedict.addConstraint(tn, 'a.b.c.d.v', [1], "not 1")

        self.assertRaises(ValueError, f)

        # control
        treedict.addConstraint(tn, 'a.b', [1], "not 1")

    def testConstraint_22_IntermediatesMustBeBranches_03_global(self):
        tn = unique_name()
        bn = unique_name()
        
        treedict.addGlobalConstraint(bn + '.a.b.c.d.v', [1], "not 1")

        p = treedict.getTree(tn)

        def f():
            p[bn + ".a.b"] = 1

        self.assertRaises(ValueError, f)

        # control
        p.makeBranch(bn + ".a.b")

    def testConstraint_22_IntermediatesMustBeBranches_04_global_after(self):
        bn = unique_name()
        tn = unique_name()
        
        p = treedict.getTree(tn)

        p[bn + ".a.b"] = 1

        def f():
            treedict.addGlobalConstraint(bn + '.a.b.c.d.v', [1], "not 1")

        self.assertRaises(ValueError, f)

        treedict.addGlobalConstraint(bn + '.a.b', [1], "not 1")

    def testConstraint_23_IntermediatesMustBeBranches_copied_01(self):
        tn = unique_name()
        
        treedict.addConstraint(tn, 'a.b.c.d.v', [1], "not 1")

        pb = treedict.getTree(tn)

        pb.makeBranch('a.b')

        p = pb.a.b.copy()

        def f():
            p["c.d"] = 1

        self.assertRaises(ValueError, f)

        # control
        p.c.makeBranch("d")

    def testConstraint_23_IntermediatesMustBeBranches_copied_02_after(self):
        tn = unique_name()
        
        pb = treedict.getTree(tn)

        pb.makeBranch('a.b')
        p = pb.a.b.copy()

        p["c.d"] = 1

        def f():
            treedict.addConstraint(tn, 'a.b.c.d.v', [1], "not 1")
            
        self.assertRaises(ValueError, f)

        # control
        treedict.addConstraint(tn, 'a.b.c.d', [1], "not 1")

    def testConstraint_23_IntermediatesMustBeBranches_copied_03_global(self):
        tn = unique_name()
        bn = unique_name()
        
        treedict.addGlobalConstraint(bn + '.a.b.c.d.v', [1], "not 1")

        pb = treedict.getTree(tn)

        pb.makeBranch(bn)
        pb[bn].makeBranch('a.b')

        p = pb[bn].a.b.copy()

        def f():
            p["c.d"] = 1

        self.assertRaises(ValueError, f)

        # control
        p.c.makeBranch("d")

    def testConstraint_23_IntermediatesMustBeBranches_copied_04_global_after(self):
        tn = unique_name()
        bn = unique_name()
        
        pb = treedict.getTree(tn)

        pb.makeBranch(bn)
        pb[bn].makeBranch('a.b')

        p = pb[bn].a.b.copy()

        p["c.d"] = 1

        def f():
            treedict.addGlobalConstraint(bn + '.a.b.c.d.v', [1], "not 1")
            
        self.assertRaises(ValueError, f)

        # control
        treedict.addGlobalConstraint(bn + '.a.b.c.d', [1], "not 1")

    
    def testConstraint_24_CopyingBranchTracks(self):
        # A regression test

        tn = unique_name()

        p = treedict.getTree(tn)

        treedict.addConstraint(tn, 'a.b.c.d.v', [1], "not 1")

        p.a.b.c.d.v = 1
        
        pc = p.a.b.copy()
        
        # would raise an error earlier
        pc.a.b.v = 1

    ##################################################
    # Handling links -- kinda a pain

    def testConstraint_25_Links_01(self):
        tn = unique_name()
        p = treedict.getTree(tn)
        treedict.addConstraint(tn, 'a.b.link.v', [1], "not 1")

        p.d.v = 1
        p.a.b.link = p.d

        self.assert_(p.a.b.link.v == 1)


    def testConstraint_25_Links_02_bad_value(self):
        tn = unique_name()
        p = treedict.getTree(tn)
        treedict.addConstraint(tn, 'a.b.link.v', [1], "not 1")

        p.d.v = 2

        def f():
            p.a.b.link = p.d
        
        self.assertRaises(ValueError, f)


    def testConstraint_25_Links_03_set_in_linked_root(self):
        tn = unique_name()
        p = treedict.getTree(tn)
        treedict.addConstraint(tn, 'a.b.link.v', [1], "not 1")

        p.a.b.link = p.d

        def f():
            p.d.v = 1

        self.assertRaises(ValueError, f)

    def testConstraint_26_global_Links_01(self):
        tn = unique_name()
        bn = unique_name()

        link_key = bn + '.a.b.link'

        p = treedict.getTree(tn)
        treedict.addConstraint(tn, link_key + '.v', [1], "not 1")

        p[link_key] = p.d
        p.d.v = 1

        self.assert_(p[link_key].v == 1)

    def testConstraint_26_global_Links_02_bad_value(self):
        tn = unique_name()
        bn = unique_name()

        link_key = bn + '.a.b.link'

        p = treedict.getTree(tn)
        treedict.addConstraint(tn, link_key + '.v', [1], "not 1")

        p[link_key] = p.d

        def f():
            p.d.v = 2

        self.assertRaises(ValueError, f)

    def testConstraint_26_global_Links_03_set_in_linked_root(self):
        tn = unique_name()
        bn = unique_name()

        link_key = bn + '.a.b.link'

        p = treedict.getTree(tn)
        treedict.addConstraint(tn, link_key + '.v', [1], "not 1")

        p.d.v = 2

        def f():
            p[link_key] = p.d

        self.assertRaises(ValueError, f)

    def testConstraint_27_after_Links_01(self):
        tn = unique_name()
        p = treedict.getTree(tn)
        
        p.d.v = 1
        p.a.b.link = p.d

        treedict.addConstraint(tn, 'a.b.link.v', [1], "not 1")

        self.assert_(p.a.b.link.v == 1)


    def testConstraint_27_after_Links_02_bad_value(self):
        tn = unique_name()
        p = treedict.getTree(tn)
        
        p.a.b.link = p.d
        p.d.v = 2

        def f():
            treedict.addConstraint(tn, 'a.b.link.v', [1], "not 1")
        
        self.assertRaises(ValueError, f)

    def testConstraint_28_after_global_Links_01(self):
        tn = unique_name()
        bn = unique_name()

        link_key = bn + '.a.b.link'

        p = treedict.getTree(tn)

        p[link_key] = p.d
        p.d.v = 1

        treedict.addConstraint(tn, link_key + '.v', [1], "not 1")

        self.assert_(p[link_key].v == 1)

    def testConstraint_28_after_global_Links_02_bad_value(self):
        tn = unique_name()
        bn = unique_name()

        link_key = bn + '.a.b.link'

        p = treedict.getTree(tn)

        p[link_key] = p.d
        p.d.v = 2

        def f():
            treedict.addConstraint(tn, link_key + '.v', [1], "not 1")

        self.assertRaises(ValueError, f)


    def testConstraint_29_Links_deeper_reference(self):
        tn = unique_name()
        p = treedict.getTree(tn)
        treedict.addConstraint(tn, 'l1.v', [1], "not 1")

        p.l1 = p.l2
        p.l2 = p.l3
        p.l3 = p.l4
        p.l4 = p.l5
        p.l5 = p.l6
        p.l6 = p.l7

        p.l7.v = 1

        self.assert_(p.l1.v == 1)
        self.assert_(p.l2.v == 1)
        self.assert_(p.l3.v == 1)
        self.assert_(p.l4.v == 1)
        self.assert_(p.l5.v == 1)
        self.assert_(p.l6.v == 1)
        self.assert_(p.l7.v == 1)
        

    def testConstraint_30_Link_Conf_01(self):
        
        tn = unique_name()
        p = treedict.getTree(tn)

        treedict.addConstraint(tn, 'a.b.c', [1], "not 1")

        p.aa.a.bb.b.c = 2
        p.a = p.aa.a

        def f():
           p.a.b = p.a.bb.b 

        self.assertRaises(ValueError, f)


    def testConstraint_30_Link_Conf_02(self):
        
        tn = unique_name()
        p = treedict.getTree(tn)

        treedict.addConstraint(tn, 'a.v', [1], "not 1")

        p.aa.bb.cc.dd.v = 2
        p.aa.bb.c = p.aa.bb.cc
        p.aa.c = p.aa.bb.c
        p.c = p.aa.c

        def f():
            p.a = p.c

        self.assertRaises(ValueError, f)

    def testConstraint_30_Link_Conf_03(self):
        
        tn = unique_name()
        p = treedict.getTree(tn)

        treedict.addConstraint(tn, 'a.v', [1], "not 1")

        p.aa.bb.cc.dd.v = 2
        p.aa.bb.c = p.aa.bb.cc
        p.aa.c = p.aa.bb.c
        p.a = p.c

        def f():
            p.c = p.aa.c
            

        self.assertRaises(ValueError, f)

    def testConstraint_30_Link_Conf_04(self):
        
        tn = unique_name()
        p = treedict.getTree(tn)

        treedict.addConstraint(tn, 'a.v', [1], "not 1")

        p.aa.bb.cc.dd.v = 2
        p.aa.bb.c = p.aa.bb.cc
        p.c = p.aa.c
        p.a = p.c

        def f():
            p.aa.c = p.aa.bb.c
            

        self.assertRaises(ValueError, f)

    def testConstraint_30_Link_Conf_05(self):
        
        tn = unique_name()
        p = treedict.getTree(tn)

        treedict.addConstraint(tn, 'a.v', [1], "not 1")

        p.aa.bb.cc.dd.v = 2
        p.aa.c = p.aa.bb.c
        p.c = p.aa.c
        p.a = p.c

        def f():
            p.aa.bb.c = p.aa.bb.cc
            

        self.assertRaises(ValueError, f)

    def testConstraint_30_Link_Conf_06(self):
        
        tn = unique_name()
        p = treedict.getTree(tn)

        treedict.addConstraint(tn, 'a.v', [1], "not 1")

        p.c = p.aa.c
        p.aa.bb.c = p.aa.bb.cc
        p.aa.c = p.aa.bb.c
        p.a = p.c

        def f():
            p.aa.bb.cc.dd.v = 2
            
        self.assertRaises(ValueError, f)

    def testConstraint_30_Link_Conf_01_control(self):
        
        tn = unique_name()
        p = treedict.getTree(tn)

        treedict.addConstraint(tn, 'a.b.c', [1], "not 1")

        p.aa.a.bb.b.c = 1
        p.a = p.aa.a

        def f():
           p.a.b = p.a.bb.b 


    def testConstraint_30_Link_Conf_02_control(self):
        
        tn = unique_name()
        p = treedict.getTree(tn)

        treedict.addConstraint(tn, 'a.v', [1], "not 1")

        p.aa.bb.cc.dd.v = 1
        p.aa.bb.c = p.aa.bb.cc
        p.aa.c = p.aa.bb.c
        p.c = p.aa.c

        def f():
            p.a = p.c

    def testConstraint_30_Link_Conf_03_control(self):
        
        tn = unique_name()
        p = treedict.getTree(tn)

        treedict.addConstraint(tn, 'a.v', [1], "not 1")

        p.aa.bb.cc.dd.v = 1
        p.aa.bb.c = p.aa.bb.cc
        p.aa.c = p.aa.bb.c
        p.a = p.c

        def f():
            p.c = p.aa.c
            

    def testConstraint_30_Link_Conf_04_control(self):
        
        tn = unique_name()
        p = treedict.getTree(tn)

        treedict.addConstraint(tn, 'a.v', [1], "not 1")

        p.aa.bb.cc.dd.v = 1
        p.aa.bb.c = p.aa.bb.cc
        p.c = p.aa.c
        p.a = p.c

        def f():
            p.aa.c = p.aa.bb.c
            

    def testConstraint_30_Link_Conf_05_control(self):
        
        tn = unique_name()
        p = treedict.getTree(tn)

        treedict.addConstraint(tn, 'a.v', [1], "not 1")

        p.aa.bb.cc.dd.v = 1
        p.aa.c = p.aa.bb.c
        p.c = p.aa.c
        p.a = p.c

        def f():
            p.aa.bb.c = p.aa.bb.cc
            

    def testConstraint_30_Link_Conf_06_control(self):
        
        tn = unique_name()
        p = treedict.getTree(tn)

        treedict.addConstraint(tn, 'a.v', [1], "not 1")

        p.c = p.aa.c
        p.aa.bb.c = p.aa.bb.cc
        p.aa.c = p.aa.bb.c
        p.a = p.c

        def f():
            p.aa.bb.cc.dd.v = 1

    def testConstraint_30_Link_Conf_01_after(self):
        
        tn = unique_name()
        p = treedict.getTree(tn)


        p.aa.a.bb.b.c = 2
        p.a.b = p.a.bb.b 
        p.a = p.aa.a

        def f():
            treedict.addConstraint(tn, 'a.b.c', [1], "not 1")

        self.assertRaises(ValueError, f)


    def testConstraint_30_Link_Conf_02_after(self):
        
        tn = unique_name()
        p = treedict.getTree(tn)
        
        p.aa.bb.cc.dd.v = 2
        p.aa.bb.c = p.aa.bb.cc
        p.aa.c = p.aa.bb.c
        p.a = p.c
        p.c = p.aa.c

        def f():
            treedict.addConstraint(tn, 'a.v', [1], "not 1")

        self.assertRaises(ValueError, f)


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python

# Copyright (c) 2009, Hoyt Koepke (hoytak@gmail.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     - Neither the name 'treedict' nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Hoyt Koepke ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Hoyt Koepke BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import random, unittest, cPickle, collections
from treedict import TreeDict, getTree
import treedict
from copy import deepcopy, copy

from hashlib import md5
import random

from common import *

class TestAttachPop(unittest.TestCase):

    def testAttaching_01(self):
        p1 = TreeDict('root')
        p2 = TreeDict('node')

        p2.a = 123
        p1.attach(p2)

        self.assert_(p1.node.a == 123)
        self.assert_(p1.node.rootNode() is p1)
        self.assert_(p2.rootNode() is p2)

    def testAttaching_02(self):
        p1 = TreeDict('root')
        p2 = TreeDict('node')

        p2.a = 123
        p1.attach(p2, copy=False)

        self.assert_(p1.node.a == 123)
        self.assert_(p1.node.rootNode() is p1)
        self.assert_(p1.node is p2)
        self.assert_(p2.rootNode() is p1)

    def testAttaching_03(self):
        p1 = TreeDict('root')
        p2 = TreeDict('node')

        p2.a = 123
        p1.attach('attachnode', p2)

        self.assert_(p1.attachnode.a == 123)
        self.assert_(p1.attachnode.rootNode() is p1)
        self.assert_(p1.attachnode == p2)
        self.assert_(p1.attachnode.branchName(True,True) == "root.attachnode")
        self.assert_(p2.rootNode() is p2)

    def testAttaching_04(self):
        p1 = TreeDict('root')
        p2 = TreeDict('node')

        p2.a = 123
        p1.attach('attachnode', p2, copy=False)

        self.assert_(p1.attachnode.a == 123)
        self.assert_(p1.attachnode.rootNode() is p1)

        self.assert_(p1.attachnode == p2)
        self.assert_(p1.attachnode.branchName(True,True) == "root.attachnode")
        self.assert_(p2.branchName(True,True) == "root.attachnode")
        self.assert_(p2.rootNode() is p1)

    def testAttaching_05(self):
        p1 = TreeDict('root')
        p1.a.b = 123
        
        self.assertRaises(ValueError, lambda: p1.a.attach(p1))

    def testAttaching_06(self):
        p1 = TreeDict('root')
        p1.a.b.c = 123
        
        def f(): p1.attach(p1.a.b, copy=False)

        self.assertRaises(ValueError, f)

    def testAttaching_07(self):
        p1 = TreeDict('root')
        p1.a.n.v = 123
        p1.b.n.v = 421
        
        p1.a.attach('m', p1.b.n)

    def testAttaching_08(self):
        
        p1 = TreeDict('root')
        p1.a.n.v = 123
        p1.b.n.v = 421
        
        def f(): p1.a.attach(p1.a.n, copy=False)
        
        self.assertRaises(ValueError, f)

    def testAttaching_09_frozen_1(self):
        p = TreeDict('p')
        p.b.c = 1
        p.freeze()

        p2 = TreeDict()
        p2.attach(p, copy=False)
        
        self.assert_(p2.p.b.c == 1)
        self.assert_(p2.p.isFrozen())
        self.assert_(p2.p.b.isFrozen())
        self.assert_(not p2.isFrozen())

    def testAttaching_09_frozen_2(self):
        p = TreeDict('p')
        p.b.c = 1
        p.freeze()

        p2 = TreeDict()
        p2.attach(p, copy=True)
        
        self.assert_(p2.p.b.c == 1)
        self.assert_(not p2.p.isFrozen())
        self.assert_(not p2.p.b.isFrozen())
        self.assert_(not p2.isFrozen())

    def testAttaching_10_Atomic_01_noImplicitOverwrite(self):
        p = TreeDict()
        p.a.b = 1

        self.assertRaises(TypeError, lambda: p.attach("a.b.c.d", TreeDict(x = 1),
                                                      protect_structure=True))

        self.assert_(p.a.b == 1)
        
    def testAttaching_10_Atomic_02_noImplicitOverwrite(self):
        p = TreeDict()
        p.makeBranch('a')
        p.freeze()

        self.assert_('a' in p.keys(branch_mode = 'only', recursive=False))

        self.assertRaises(TypeError, lambda: p.attach("a.b.c.d", TreeDict(x = 1),
                                                      protect_structure=True))

        self.assert_('a' in p.keys(branch_mode = 'only', recursive=False))
        
        self.assert_(p.a.size() == 0)
        
    def testAttaching_11_BadParameters_01(self):

        p = TreeDict()

        self.assertRaises(TypeError, lambda: p.attach(TreeDict(), "name"))

    def testAttaching_11_BadParameters_02(self):
        p = TreeDict()
        self.assertRaises(TypeError, lambda: p.attach(TreeDict(), TreeDict()))

    def testAttaching_11_BadParameters_03(self):
        p = TreeDict()
        self.assertRaises(TypeError, lambda: p.attach(copy = True))

    def testAttaching_11_BadParameters_04(self):
        p = TreeDict()
        self.assertRaises(TypeError, lambda: p.attach(copy = False))

    def testRecursiveAttach_01(self):
        p1 = TreeDict('root')
        p1.b1 = TreeDict('new_branch')
        p1.b1.b2 = TreeDict('new_branch')
        p1.b1.b2.b3 = TreeDict('new_branch')

        self.assert_(p1.b1.isRoot())
        self.assert_(p1.b1.b2.isRoot())
        self.assert_(p1.b1.b2.b3.isRoot())
        self.assert_(p1.b1.branchName(True,True) == 'new_branch')
        self.assert_(p1.b1.b2.branchName(True,True) == 'new_branch')
        self.assert_(p1.b1.b2.b3.branchName(True,True) == 'new_branch')

        p1.attach(copy=True, recursive=True)
        
        self.assert_(p1.b1.rootNode() is p1)
        self.assert_(p1.b1.b2.rootNode() is p1)
        self.assert_(p1.b1.b2.b3.rootNode() is p1)
        self.assert_(p1.b1.branchName(True,True) == 'root.b1')
        self.assert_(p1.b1.b2.branchName(True,True) == 'root.b1.b2')
        self.assert_(p1.b1.b2.b3.branchName(True,True) == 'root.b1.b2.b3')

    def testRecursiveAttach_02(self):
        p1 = TreeDict('root')
        p1.b1 = TreeDict('new_branch')
        p1.b1.b2 = TreeDict('new_branch')
        p1.b1.b2.b3 = TreeDict('new_branch')

        self.assert_(p1.b1.isRoot())
        self.assert_(p1.b1.b2.isRoot())
        self.assert_(p1.b1.b2.b3.isRoot())

        self.assert_(p1.b1.branchName(True,True) == 'new_branch')
        self.assert_(p1.b1.b2.branchName(True,True) == 'new_branch')
        self.assert_(p1.b1.b2.b3.branchName(True,True) == 'new_branch')

        p1.attach(copy=False, recursive=True)
        
        self.assert_(p1.b1.rootNode() is p1)
        self.assert_(p1.b1.b2.rootNode() is p1)
        self.assert_(p1.b1.b2.b3.rootNode() is p1)

        self.assert_(p1.b1.branchName(True,True) == 'root.b1')
        self.assert_(p1.b1.b2.branchName(True,True) == 'root.b1.b2')
        self.assert_(p1.b1.b2.b3.branchName(True,True) == 'root.b1.b2.b3')

    def testRecursiveAttach_03_recursive_with_linked_nodes(self):
        p = TreeDict()

        p.a = p.adef.a
        p.adef.a.v = 1

        p.attach(recursive=True)

    def testRecursiveAttaching_04_error_on_frozen(self):
        
        p = TreeDict()
        p.b = TreeDict(x = 1)
        p.freeze()

        self.assertRaises(TypeError, lambda: p.attach(recursive = True))
                         
        self.assert_(p.b.isRoot())
        self.assert_(p.b not in p.branches())
        self.assert_(p.size() == 1)


    def testAttaching_10_EqualityTestingWithRecursiveAttach(self):
        
        p1 = TreeDict('root')
        p1.b1 = TreeDict('new_branch')
        p1.b1.b2 = TreeDict('new_branch')
        p1.b1.b2.b3 = TreeDict('new_branch')

        p2 = p1.copy()

        self.assert_(p2 == p1)

        p1.attach(copy=False, recursive=True)
        
        self.assert_(p2 != p1)

    def testAttaching_11_equalityTesting_01(self):

        p1 = TreeDict('root')
        p1.b1 = TreeDict('new_branch')
        p1.b1.b2 = TreeDict('new_branch')
        p1.b1.b2.b3 = TreeDict('new_branch')

        p2 = p1.copy()

        self.assert_(p2 == p1)

        p1.attach(copy=False, recursive=True)
        
        self.assert_(p2 != p1)

    def testAttaching_12_defaultName_RecursiveAttach(self):
        p1 = TreeDict()
        p1.p2 = TreeDict()
        
        p1.attach(recursive = True)


    def testDetaching_01(self):
        p1 = TreeDict('root')
        p1.node.a = 123
        p1.node.b = "12312321"
        p1.node2.a = 13
        p1.node3.a = "123312"
        p1.node4 = 123
        
        p1copy = deepcopy(p1)

        p2 = p1.node
        
        p3 = p1.pop("node")

        self.assert_(p2 is p3)
        self.assert_(p3.rootNode() is p3)
        self.assert_("node" not in p1)
        self.assert_(p2 == p1copy.node)
        self.assert_(p1 != p1copy)


    def testDetaching_02(self):

        # This covers the corner case of no elements remaining.
        p1 = TreeDict('root')
        p1.node.a = 123
        p1copy = deepcopy(p1)

        p2 = p1.node
        
        p3 = p1.pop("node")

        self.assert_(p2 is p3)
        self.assert_(p3.rootNode() is p3)
        self.assert_("node" not in p1)
        self.assert_(p2 == p1copy.node)
        self.assert_(p1 != p1copy)

    def testDetaching_03_Dangling_AttributeError(self):
        
        p = TreeDict()

        def f():
            q = p.a.b.pop()

        self.assertRaises(AttributeError, f)

    def testDetaching_04_dangling_AttributeError_correct_node(self):
        
        p = TreeDict()
        ae_msg = ''

        try:
            q = p.aduvulksjucmfiddkjdo.b.pop()
        except AttributeError, ae:
            ae_msg = str(ae)

        # Make sure it propegates up so the error message is sent from
        # the root dangling node.
        self.assert_('aduvulksjucmfiddkjdo' in ae_msg)


    def testDetach_05_self(self):
        p = TreeDict()
        
        p.a.b.c.d = 1

        b = p.a.b

        bd = p.a.b.pop()

        self.assert_(b is bd)
        self.assert_('a' in p)
        self.assert_('a.b' not in p)
        self.assert_(b.parentNode() == None)


    def testDetach_06_complex_key(self):
        p = TreeDict()
        p.a.b.c.d = 1
        b = p.a.b

        bd = p.pop('a.b')
        
        self.assert_(b is bd)
        self.assert_('a' in p)
        self.assert_('a.b' not in p)
        self.assert_(b.parentNode() == None)

    def testDetach_07_nonbranch(self):
        # Test to make sure the behavior is like pop() (perhaps we
        # should chang the name ??)

        p = TreeDict()
        p.a = 1

        v = p.pop('a')
        
        self.assert_(v == 1)
        self.assert_('a' not in p)


    def testDetach_08_nonbranch_complex(self):
        # Test to make sure the behavior is like pop() (perhaps we
        # should chang the name ??)

        p = TreeDict()
        p.a.b.c = 1

        v = p.pop('a.b.c')
        
        self.assert_(v == 1)
        self.assert_('a' in p)
        self.assert_('a.b' in p)
        self.assert_('a.b.c' not in p)
        self.assert_('c' not in p.a.b)

    def testClear_01_all(self):
        p, rid = basic_walking_test()
        p = p.copy()
        p.clear("all")

        self.assert_(len(p) == 0)

    def testClear_02_branches_only(self):
        p, rid = basic_walking_test()
        p = p.copy()

        p.clear("only")

        self.assert_(len(p) == len(rid[(False, "none")]))

        for k, v in rid[(False, "none")]:
            self.assert_(k in p)

    def testClear_03_branches_none(self):
        p, rid = basic_walking_test()

        p = p.copy()

        p.clear("none")

        self.assert_(len(p) == len(rid[(False, "only")]))

        for k, v in rid[(False, "only")]:
            self.assert_(k in p)

    def testClear_04_IsFrozenError(self):

        p = sample_tree()

        p.freeze()

        self.assertRaises(TypeError, lambda: p.clear())

    def testClear_05_IsDanglingError(self):
        
        p = sample_tree()

        self.assertRaises(AttributeError, lambda: p.a.clear())


class TestBranches(unittest.TestCase):

    def testBranch_01_basic(self):
        p = TreeDict()
        p.makeBranch('b')
        self.assert_('b' in p)
        self.assert_(not p.b.isDangling())

    def testBranch_02_ReturnsCorrectBranch(self):
        p = TreeDict()
        b = p.makeBranch('b')

        self.assert_(p.b is b)

    def testBranch_02b_ReturnsCorrectBranch(self):
        p = TreeDict()
        b = p.makeBranch('a').makeBranch('b')
        
        self.assert_(p.a.b is b)

    def testBranch_02c_ReturnsCorrectBranch(self):
        # Regression

        p = TreeDict()
        a = p.makeBranch('a')
        p.a.b
        
        self.assert_(p.a is a)

    def testBranch_03_KeyedValues(self):
        p = TreeDict()
        b = p.makeBranch('a.b.c')

        self.assert_(p.a.b.c is b)

    def testBranch_04_ReturnsCorrectBranch(self):
        p = TreeDict()
        b = p.makeBranch('b')

        self.assert_(p.b is b)

    def testBranch_05_Overwrite(self):
        # Decided this is okay

        p = TreeDict()
        p.d.a = 1
        p.d = 1    


    def testBranch_05_BranchFunction(self):
        p = TreeDict()

        b = p.makeBranch("a")

        self.assert_(b is p.a)

    def testBranch_06_Branch_Sets_Dangling(self):

        p = TreeDict()

        b = p.a

        self.assert_("a" not in p)
        self.assert_(p.a.isDangling())

        b.makeBranch("b")

        self.assert_(not p.a.isDangling())
        self.assert_("a.b" in p)

    def testBranch_07_TreeDict_Values_Not_Frozen(self):

        p = TreeDict()
        p.a.x = 1
        p.c.t = TreeDict()
        p.c.t.x = 1

        self.assert_(not p.isFrozen())
        self.assert_(not p.a.isFrozen())
        self.assert_(not p.c.isFrozen())
        self.assert_(not p.c.t.isFrozen())

        p.freeze()

        self.assert_(p.isFrozen())
        self.assert_(p.a.isFrozen())
        self.assert_(p.c.isFrozen())
        self.assert_(not p.c.t.isFrozen())

class TestDangling(unittest.TestCase):

    def testDangling_01_BranchOverwrite(self):

        p1 = sample_tree()
        p1.a

        self.assert_("a" not in p1)

        p1.a.b = 2

        self.assert_("a" in p1)
        self.assert_(p1.a.b == 2)

    def testDangling_02_NotRetrieved(self):
        self.assertRaises(KeyError, lambda: TreeDict()["a"])

    
    def testDangling_03_OutOfOrderSetting(self):
        p = TreeDict()

        c = p.a.c
        d = p.a.d
        
        c.v = 1
        d.v = 2

        self.assert_('a.c.v' in p)
        self.assert_('a.d.v' in p)
        
        self.assert_(p.a.c.v == 1)
        self.assert_(p.a.d.v == 2)


    def testDangling_04_OutOfOrderSetting_same_names(self):
        p = TreeDict()

        c1 = p.a.c
        c2 = p.a.c

        self.assert_(c1 is c2)
        
        c1.v = 1
        c2.v = 2

        self.assert_('a.c.v' in p)
        self.assert_(p.a.c.v == 2)

    def testDangling_05_Retrieval(self):
        # tests to make sure that dangling nodes are retrieved properly

        p = TreeDict()

        self.assert_(p.a is p.a)


    def testDangling_06_Retrieval_SecondOrder(self):
        # tests to make sure that dangling nodes are retrieved properly

        p = TreeDict()

        self.assert_(p.a.b is p.a.b)

    def testDangling_07_Existance_01(self):
        p = TreeDict()

        p.a

        self.assert_('a' not in p)

    def testDangling_08_Freezing_01(self):
        p = TreeDict()

        b = p.a.b

        p.freeze()

        def f():
            b.v = 1

        self.assertRaises(TypeError, f)

    def testDangling_08_Freezing_02(self):
        p = TreeDict()

        b = p.a.b

        del p['a']

        p.freeze()

        # Okay now as it's disconnected from b
        b.v = 1

    def testDangling_09_Count_01(self):
        p = TreeDict()
        self.assert_(p._numDangling() == 0)
        p.a
        self.assert_(p._numDangling() == 1)

    def testDangling_09_Count_02(self):
        p = TreeDict()
        self.assert_(p._numDangling() == 0)
        p.a.b
        self.assert_(p._numDangling() == 1)

    def testDangling_09_Count_03_Deletion(self):
        p = TreeDict()
        self.assert_(p._numDangling() == 0)
        p.a
        self.assert_(p._numDangling() == 1)
        del p.a
        self.assert_(p._numDangling() == 0)

    def testDangling_09_Count_03b_Deletion(self):
        p = TreeDict()
        self.assert_(p._numDangling() == 0)
        p.a
        self.assert_(p._numDangling() == 1)
        del p["a"]
        self.assert_(p._numDangling() == 0)

    def testDangling_09_Count_04_Branching(self):
        p = TreeDict()
        self.assert_(p._numDangling() == 0)
        p.a
        self.assert_(p._numDangling() == 1)
        p.makeBranch("a")
        self.assert_(p._numDangling() == 0)

    def testDangling_09_Count_05_Branching(self):
        p = TreeDict()
        p.makeBranch("a")
        self.assert_(p._numDangling() == 0)

    def testDangling_09_Count_06_Branching(self):
        p = TreeDict()
        p.makeBranch("a")
        p.a.b
        self.assert_(p._numDangling() == 0)
        self.assert_(p.a._numDangling() == 1)
        
    def testDangling_10_ForwardReferenceSetsDangling(self):
        p = TreeDict()

        v = "ASDFFEVgdkshvfnjdsnfojsanjierf"

        p.l1 = p.l2
        p.l2.n = p.l3
        p.l3.n = p.l4
        p.l4.n = p.l5
        p.l5.n = p.l6
        p.l6.n = p.l7
        p.l7.v = v

        self.assert_(p.l7.v is v)
        self.assert_(p.l6.n.v is v)
        self.assert_(p.l5.n.n.v is v)
        self.assert_(p.l4.n.n.n.v is v)
        self.assert_(p.l3.n.n.n.n.v is v)
        self.assert_(p.l2.n.n.n.n.n.v is v)
        self.assert_(p.l1.n.n.n.n.n.v is v)

    def testDangling_11_ReferenceEquivalence_01(self):

        p = TreeDict()

        b = p.a = p.b

        self.assert_(p.a is p.b)

        p.b = 1

        self.assert_(p.a is b)

    def testDangling_11_ReferenceEquivalence_02(self):

        p = TreeDict()

        a1 = p.a1 = p.b
        a2 = p.a2 = p.b

        self.assert_(p.a1 is p.b)
        self.assert_(p.a2 is p.b)
        self.assert_(a1 is a2)

        p.b = 1

        self.assert_(p.a1 is a1)
        self.assert_(p.a2 is a1)

    def testDangling_12_ForwardAssingmentSetCorrectlyWithReference(self):

        p = TreeDict()
        v = unique_object()
        
        b = p.a = p.c
        p.c = v

        self.assert_(p.a is not v)
        self.assert_(p.a.isDangling() )

        p.a.v = v

        self.assert_(not p.a.isDangling() )
        self.assert_(p.a.v is v)
        self.assert_(b.v is v)
        self.assert_(b is p.a)
        self.assert_(p.c is v)

    def testDangling_13_CorrectCountsThroughSiblingReferences(self):

        p = TreeDict()

        p.a = p.d
        self.assert_(p._numDangling() == 2)

        # Overwrite p.d; p.a still points to the dangling node, so the
        # count should not go down.

        p.d = 1
        self.assert_(p._numDangling() == 1)
        
    def testDangling_14_CorrectParentingSetAtFixing_01(self):

        p = TreeDict()
        v = unique_object()

        p.b = p.c
        p.a = p.b

        # Set c and fix p.b
        p.c = 1
        p.b.x = 1

        # Now b should be 
        self.assert_(p.b.branchName() == 'b')
        self.assert_(p.a is p.b)

    def testDangling_14_CorrectParentingSetAtFixing_02(self):

        p = TreeDict()
        v = unique_object()

        p.b = p.c
        p.a = p.b

        # Set c and fix p.b
        p.c = 1
        p.a.x = 1

        # Now b should be 
        self.assert_(p.b.branchName() == 'b')
        self.assert_(p.a is p.b)

    def testDangling_14_CorrectParentingSetAtFixing_03(self):

        p = TreeDict()
        v = unique_object()

        p.b = p.c
        p.a = p.b

        # Set c and fix p.b
        p.c = 1
        p.b = 1
        p.a.x = 1

        # Now b should be 
        self.assert_(p.a.branchName() == 'a')
        
    def testDangling_14_CorrectParentingSetAtFixing_04(self):

        p = TreeDict()
        v = unique_object()

        p.makeBranch('cc')
        
        p.b = p.cc.c
        p.a = p.b

        # Set c and fix p.b
        p.cc.c = 1
        p.b.x = 1

        # Now b should be 
        self.assert_(p.b.branchName() == 'b')
        self.assert_(p.a is p.b)

    def testDangling_14_CorrectParentingSetAtFixing_05(self):

        p = TreeDict()
        v = unique_object()

        p.makeBranch('cc')
        
        p.b = p.cc.c
        p.aa.a = p.b

        # Set c and fix p.b
        p.cc.c = 1
        p.b = 1

        # Now b should be 
        self.assert_(p.aa.a.branchName() == 'a')

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python

# Copyright (c) 2009, Hoyt Koepke (hoytak@gmail.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     - Neither the name 'treedict' nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Hoyt Koepke ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Hoyt Koepke BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import random, unittest, cPickle, collections
from treedict import TreeDict, getTree
import treedict
from copy import deepcopy, copy

from hashlib import md5
import random

from treedict.treedict import _ldist

from common import *

class TestRetrieval(unittest.TestCase):
       
    def test_existance_01(self):
        p = sample_tree()
        self.assert_("123" not in p)
        self.assert_(not p.has_key("123"))

    def test_existance_02(self):
        p = sample_tree()
        self.assert_(123 not in p)
        self.assert_(not p.has_key(123))

    def test_existance_03(self):
        p = sample_tree()
        self.assert_(None not in p)
        self.assert_(not p.has_key(None))

    def test_existance_04(self):
        p = sample_tree()
        self.assert_("si3dkdkdmmd" not in p)
        self.assert_(not p.has_key("si3dkdkdmmd"))

    def test_existance_05(self):
        p = sample_tree()
        self.assert_(p not in p)
        self.assert_(not p.has_key(p))


    def test_existance_06_dangling_node(self):
        p = TreeDict('roor')
        p.a

        self.assert_('a' not in p)

    def test_existance_06b_dangling_node(self):
        p = TreeDict('roor')

        p.b = 123
        p.a

        self.assert_('b' in p)
        self.assert_('a' not in p)

    def test_existance_06c_dangling_node(self):
        p = TreeDict('roor')

        p.b = 123
        p.a
        p.aa.b.c
        p.bb.c.d = None

        self.assert_('a' not in p)
        self.assert_('b' in p)
        self.assert_('aa' not in p)
        self.assert_('bb' in p)

    def testExistanceThroughLink(self):
        p = TreeDict()
        p.a.b.link = p.d
        p.d.v = 1
        
        self.assert_('a.b.link.v' in p)
            
    def testContains_01(self):
        p1 = TreeDict()
        p1.a.b = 123
        
        self.assert_('a' in p1)
        self.assert_('a.b' in p1)
        self.assert_('b' not in p1)
        self.assert_('b' in p1.a)


    def testContains_02(self):
        p1 = TreeDict()
        p1.a.b = 123
        p1.d
        
        self.assert_('a' in p1)
        self.assert_('a.b' in p1)
        self.assert_('d' not in p1)

    def testForwardReference(self):
        p = TreeDict('test')

        p.a.b.c = p.d.e.f

        p.d.e.f.g = 10

        self.assert_(p.a.b.c.g == 10)

    def testDefaultValue_01(self):
        p = TreeDict()
        p.a.b = 123

        self.assert_(p.get('a.b') == 123)
        self.assert_(p.get('a.b', default_value = 1) == 123)
        self.assert_(p.get('a.c', default_value = 1) == 1)
        self.assert_(p.get('a.c', default_value = None) is None)        


    def testRetrieve_01_NonExistantBranchFromFrozenTree(self):
        p = TreeDict()
        p.a.b.c = 1
        p.freeze()

        self.assert_(p.a.b.c == 1)

        self.assertRaises(AttributeError, lambda: p.a.d)

    def testRetrieve_02_NonExistantBranchFromFrozenTree_control(self):
        p = TreeDict()
        p.a.b.c = 1
        # control; no freeze

        self.assert_(p.a.b.c == 1)
        
        p.a.d

    def testRetrieve_03_ThroughLink(self):
        p = TreeDict()
        p.a.b.link = p.d
        p.d.v = 1

        self.assert_(p["a.b.link.v"] == 1)
        self.assert_(p.get("a.b.link.v") == 1)

    def testRetrieve_04_ThroughMultipleLinks(self):
        p = TreeDict()

        p.l7.v = 1

        p.l6 = p.l7
        p.l5 = p.l6
        p.l4 = p.l5
        p.l3 = p.l4
        p.l2 = p.l3
        p.l1 = p.l2

        self.assert_(p.l7.v == 1)
        self.assert_(p.l6.v == 1)
        self.assert_(p.l5.v == 1)
        self.assert_(p.l4.v == 1)
        self.assert_(p.l3.v == 1)
        self.assert_(p.l2.v == 1)
        self.assert_(p.l1.v == 1)

    def testNonexistantValues_01(self):

        p = TreeDict()
        self.assertRaises(KeyError, lambda: p["a"])

    def testNonexistantValues_02(self):

        p = TreeDict()
        self.assertRaises(KeyError, lambda: p[0])

    def testNonexistantValues_03(self):

        p = TreeDict()
        self.assertRaises(KeyError, lambda: p[None])

    def testNonexistantValues_04(self):

        p = TreeDict()
        p.freeze()
        self.assertRaises(AttributeError, lambda: p.a)

if __name__ == '__main__':
    unittest.main()





cdef inline size_t min3(size_t a, size_t b, size_t c):
    cdef size_t m = a
    if b < m: m = b
    if c < m: m = c
    return m

cdef inline size_t min2(size_t a, size_t b):
    return a if a < b else b

cdef inline size_t max2(size_t a, size_t b):
    return b if a < b else a

cdef inline int min2_int(int a, int b):
    return a if a < b else b

cdef inline int max2_int(int a, int b):
    return b if a < b else a

cdef inline long min2_long(long a, long b):
    return a if a < b else b

cdef inline long max2_long(long a, long b):
    return b if a < b else a



# Copyright (c) 2009, Hoyt Koepke (hoytak@gmail.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     - Neither the name 'paramtree' nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Hoyt Koepke ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Hoyt Koepke BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# This pxd module just provides a centralized and fast way of handling
# size_t* buffers, which seems to happen a lot.

cdef extern from "Python.h":
    void* malloc "PyMem_Malloc" (size_t n)
    void* realloc "PyMem_Realloc" (void *p, size_t n)
    void free "PyMem_Free" (void *p)

cdef struct size_t_v:
    size_t *d
    size_t size

cdef inline size_t_v new_size_t_v(size_t size):
    cdef size_t_v v

    if size > 0:
        v.d = <size_t*>malloc(size*sizeof(size_t))
    else:
        v.d = NULL

    v.size = size
    
    return v

cdef inline void resize_size_t_v(size_t_v* v_ptr, size_t newsize):

    if newsize <= v_ptr.size:
        return
    elif v_ptr.d == NULL:
        v_ptr.d = <size_t*>malloc(newsize*sizeof(size_t))
    else:
        v_ptr.d = <size_t*>realloc(v_ptr.d, newsize*sizeof(size_t))
    
    v_ptr.size = newsize
        
cdef inline void free_size_t_v(size_t_v* v_ptr):
    if v_ptr.d != NULL:
        free(v_ptr.d)

    v_ptr.size = 0
    v_ptr.d = NULL




#!/usr/bin/env python

################################################################################
# All the control parameters should go here

source_directory_list = ['treedict']
compiler_args = []
link_args = []
version = "0.10"
description="A fast and full-featured dict-like tree container to simplify the bookkeeping surrounding parameters, variables and data."
author = "Hoyt Koepke"
author_email="hoytak@gmail.com"
name = 'treedict'
scripts = []
url = "http://www.stat.washington.edu/~hoytak/code/treedict/"
download_url = "http://pypi.python.org/pypi/treedict"

long_description = \
"""
TreeDict is a dictionary-like, hierarchical python container to
simplify the bookkeeping surrounding parameters, variables and
data. It aims to be fast, lightweight, intuitive, feature-rich and
stable.

While intended for general python development, it includes a number of
features particularly useful for scientific programming.  It is
similar in basic functionality to MATLAB structures in terms of
concise syntax and implicit branch creation.  In addition, though,
TreeDict implements all the methods of regular dictionaries, pickling,
fast non-intersecting hashing for efficient caching, simple and
advanced manipulations on the tree structure, and a system for forward
referencing branches to make lists of parameters more readable.

While still in Beta, TreeDict is well tested and extensively covered
by unit tests.
"""

classifiers = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: MacOS :: MacOS X',
    'Operating System :: Microsoft :: Windows',
    'Operating System :: POSIX',
    'Programming Language :: Python',
    'Programming Language :: Cython',
    'Topic :: Scientific/Engineering',
    'Topic :: Software Development'
    ]

# Stuff for extension module stuff
extra_library_dirs = []
extra_include_dirs = []

library_includes = []
specific_libraries = {}

################################################################################
# Shouldn't have to adjust anything below this line...

from glob import glob
import os
from os.path import split, join
from itertools import chain
import sys

from distutils.core import setup
from distutils.extension import Extension

######################################################
# First have to see if we're authorized to use cython files, or if we
# should instead compile the included files

if "--cython" in sys.argv:
    cython_mode = True
    del sys.argv[sys.argv.index("--cython")]
else:
    cython_mode = False

if "--debug" in sys.argv:
    debug_mode_c_code = True
    del sys.argv[sys.argv.index("--debug")]
else:
    debug_mode_c_code = False

# Get all the cython files in the sub directories and in this directory
if cython_mode:
    cython_files = dict( (d, glob(join(d, "*.pyx"))) for d in source_directory_list + ['.'])
else:
    cython_files = {}

all_cython_files = set(chain(*cython_files.values()))

print "+++++++++++++++++++"

if cython_mode:
    print "Cython Files Found: \n%s\n+++++++++++++++++++++" % ", ".join(sorted(all_cython_files))
else:
    print "Cython support disabled; compiling extensions from pregenerated C sources."
    print "To enable cython, run setup.py with the option --cython."
    print "+++++++++++++++++++"

# Set the compiler arguments -- Add in the environment path stuff
ld_library_path = os.getenv("LD_LIBRARY_PATH")

if ld_library_path is not None:
    lib_paths = ld_library_path.split(":")
else:
    lib_paths = []

include_path = os.getenv("INCLUDE_PATH")
if include_path is not None:
    include_paths = [p.strip() for p in include_path.split(":") if len(p.strip()) > 0]
else:
    include_paths = []


# get all the c files that are not cythonized .pyx files.
c_files   = dict( (d, [f for f in glob(join(d, "*.c"))
                       if (f[:-2] + '.pyx') not in all_cython_files])
                  for d in source_directory_list + ['.'])

for d, l in chain(((d, glob(join(d, "*.cxx"))) for d in source_directory_list + ['.']),
                  ((d, glob(join(d, "*.cpp"))) for d in source_directory_list + ['.'])):
    c_files[d] += l


print "C Extension Files Found: \n%s\n+++++++++++++++++++++" % ", ".join(sorted(chain(*c_files.values())))

# Collect all the python modules
def get_python_modules(f):
    d, m = split(f[:f.rfind('.')])
    return m if len(d) == 0 else d + "." + m

exclude_files = set(["setup.py"])
python_files = set(chain(* (list(glob(join(d, "*.py")) for d in source_directory_list) + [glob("*.py")]))) 
python_files -= exclude_files

python_modules = [get_python_modules(f) for f in python_files]

print "Relevant Python Files Found: \n%s\n+++++++++++++++++++++" % ", ".join(sorted(python_files))

if __name__ == '__main__':
    # The rest is also shared with the setup.py file, in addition to
    # this one, so 

    def strip_empty(l):
        return [e.strip() for e in l if len(e.strip()) != 0]

    def get_include_dirs(m):
        return strip_empty(extra_include_dirs + include_paths)

    def get_library_dirs(m):
        return strip_empty(extra_library_dirs + lib_paths)

    def get_libraries(m):
        return strip_empty(library_includes + (specific_libraries[m] if m in specific_libraries else []))
    
    def get_extra_compile_args(m):
        return strip_empty(compiler_args + (['-g', '-O0'] if debug_mode_c_code else []))
    
    def get_extra_link_args(m):
        return strip_empty(link_args + (['-g'] if debug_mode_c_code else []))


    ############################################################
    # Cython extension lists

    def makeExtensionList(d, filelist):
        ext_modules = []

        for f in filelist:
            f_no_ext = f[:f.rfind('.')]
            f_mod = split(f_no_ext)[1]
            modname = "%s.%s" % (d, f_mod) if d != '.' else f_mod
            
            ext_modules.append(Extension(
                    modname,
                    [f],
                    include_dirs = get_include_dirs(modname),
                    library_dirs = get_library_dirs(modname),
                    libraries = get_libraries(modname),
                    extra_compile_args = get_extra_compile_args(modname),
                    extra_link_args = get_extra_link_args(modname),
                    ))

        return ext_modules

    ############################################################
    # Now get all these ready to go

    ext_modules = []

    if cython_mode:
        from Cython.Distutils import build_ext

        ext_modules += list(chain(*list(makeExtensionList(d, l) 
                                        for d, l in cython_files.iteritems())))
        
        cmdclass = {'build_ext' : build_ext}
    else:
        cmdclass = {}

    ext_modules += list(chain(*list(makeExtensionList(d, l)
                                    for d, l in c_files.iteritems())))
    setup(
        version = version,
        description = description,
        author = author, 
        author_email = author_email,
        name = name,
        cmdclass = cmdclass,
        ext_modules = ext_modules,
        py_modules = python_modules,
        scripts = scripts,
        classifiers = classifiers,
        url = url,
        download_url = download_url)




#!/usr/bin/env python

# Copyright (c) 2009, Hoyt Koepke (hoytak@gmail.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     - Neither the name 'treedict' nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Hoyt Koepke ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Hoyt Koepke BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import random, unittest, cPickle, collections
from treedict import TreeDict, getTree
import treedict
from copy import deepcopy, copy
from itertools import chain

from hashlib import md5
import random

from common import *

class TestUpdate(unittest.TestCase):

    def checkUpdate(self, base_tree, update_tree, deep_copy,
                    check_function, overwrite, protect_structure):

        # checks it to make sure dict, iterkeys, and treedict all have the same behavior.

        p1 = base_tree.copy(deep = deep_copy)
        p2 = base_tree.copy(deep = deep_copy)
        p3 = base_tree.copy(deep = deep_copy)

        p1.update(update_tree.iteritems(), overwrite, protect_structure)
        check_function(p1)

        p2.update(dict(update_tree.iteritems()), overwrite, protect_structure)
        check_function(p2)

        prev_update_tree = update_tree.copy()

        p3.update(update_tree, overwrite, protect_structure)
        check_function(p3)

        self.assert_(prev_update_tree == update_tree)

    def checkUpdateBadStructure(self, base_tree, update_tree):

        # checks it to make sure dict, iterkeys, and treedict all have the same behavior.

        p1 = base_tree.copy()
        p2 = base_tree.copy()
        p3 = base_tree.copy()

        self.assertRaises(TypeError, lambda: p1.update(update_tree.iteritems(), True, True))
        self.assert_(p1 == base_tree)

        self.assertRaises(TypeError, lambda: p2.update(dict(update_tree.iteritems()), True, True))
        self.assert_(p2 == base_tree)

        prev_update_tree = update_tree.copy()

        self.assertRaises(TypeError, lambda: p3.update(update_tree, True, True))
        self.assert_(p3 == base_tree)

        self.assert_(prev_update_tree == update_tree)


    def testUpdate_01_basic_01(self):
        p = TreeDict(x = 1, y = 2)
        q = TreeDict(y = 3, z = 4)

        def check(r):
            self.assert_(r == TreeDict(x = 1, y = 3, z = 4))

        # This should be the same between these options
        self.checkUpdate(p, q, True, check, overwrite = True, protect_structure = False) 
        self.checkUpdate(p, q, True, check, overwrite = True, protect_structure = True) 

    def testUpdate_01_basic_02_no_overwrite(self):
        p = TreeDict(x = 1, y = 2)
        q = TreeDict(y = 3, z = 4)

        def check(r):
            self.assert_(r.y == 2)
            self.assert_(r == TreeDict(x = 1, y = 2, z = 4))

        self.checkUpdate(p, q, False, check, overwrite = False, protect_structure = False)
        self.checkUpdate(p, q, False, check, overwrite = False, protect_structure = True)

    def testUpdate_02_branches_01(self):

        p = TreeDict()
        p.a.x = 1
        p.a.y = 2
        
        q = TreeDict()
        q.a.y = 3
        q.a.z = 4

        def check(r):
            self.assert_(r.a.x == 1)
            self.assert_(r.a.y == 3)
            self.assert_(r.a.z == 4)
        
        self.checkUpdate(p, q, True, check, overwrite = True, protect_structure = False)
        self.checkUpdate(p, q, True, check, overwrite = True, protect_structure = True)

    def testUpdate_02_branches_02_no_overwrite(self):

        p = TreeDict()
        p.a.x = 1
        p.a.y = 2
        
        q = TreeDict()
        q.a.y = 3
        q.a.z = 4

        def check(r):
            self.assert_(r.a.x == 1)
            self.assert_(r.a.y == 2)
            self.assert_(r.a.z == 4)
        
        self.checkUpdate(p, q, True, check, overwrite = False, protect_structure = False)
        self.checkUpdate(p, q, True, check, overwrite = False, protect_structure = True)
        
    def testUpdate_03_StructurePreserving_01(self):
        
        p = TreeDict()
        p.a.x = 1
        p.a.y = 2
        
        q = TreeDict()
        q.a = 3

        def check(r):
            self.assert_(r == TreeDict(a = 3))

        self.checkUpdate(p, q, True, check, overwrite = True, protect_structure = False)

    def testUpdate_03_StructurePreserving_02_noOverwrite(self):
        
        p = TreeDict()
        p.a.x = 1
        p.a.y = 2

        ct = p.copy()
        
        q = TreeDict()
        q.a = 3

        def check(r):
            self.assert_(r == ct)

        self.checkUpdate(p, q, True, check, overwrite = False, protect_structure = False)
        self.checkUpdate(p, q, True, check, overwrite = False, protect_structure = True)

    def testUpdate_03_StructurePreserving_03_correctError(self):
        
        p = TreeDict()
        p.a.x = 1
        p.a.y = 2

        ct = p.copy()
        
        q = TreeDict()
        q.a = 3

        self.checkUpdateBadStructure(p, q)

    def testUpdate_04_StructurePreserving_01(self):
        
        p = TreeDict()
        p.a = 3

        q = TreeDict()
        q.a.x = 1
        q.a.y = 2

        ct = q.copy()

        def check(r):
            self.assert_(r == ct)

        self.checkUpdate(p, q, True, check, overwrite = True, protect_structure = False)

    def testUpdate_04_StructurePreserving_02_noOverwrite(self):
        
        p = TreeDict()
        p.a = 3

        q = TreeDict()
        q.a.x = 1
        q.a.y = 2

        ct = q.copy()

        def check(r):
            self.assert_(r == TreeDict(a = 3))

        self.checkUpdate(p, q, True, check, overwrite = False, protect_structure = False)
        self.checkUpdate(p, q, True, check, overwrite = False, protect_structure = True)

    ################################################################################
    # Check object preservation when we're not supposed to have copying -- toy case
    
    def checkUpdate_05_SameObjectPreserved_01_simple(self, overwrite, protect_structure):
        u = unique_object()
        p = TreeDict()
        q = TreeDict(x = u)

        def check(r):
            self.assert_(r.x is u)

        self.checkUpdate(p, q, False, check, overwrite, protect_structure)
 
    def testUpdate_05_SameObjectPreserved_01_simple_FF(self):
        self.checkUpdate_05_SameObjectPreserved_01_simple(False, False)

    def testUpdate_05_SameObjectPreserved_01_simple_FT(self):
        self.checkUpdate_05_SameObjectPreserved_01_simple(False, True)

    def testUpdate_05_SameObjectPreserved_01_simple_FF(self):
        self.checkUpdate_05_SameObjectPreserved_01_simple(True, False)

    def testUpdate_05_SameObjectPreserved_01_simple_FT(self):
        self.checkUpdate_05_SameObjectPreserved_01_simple(True, True)

    ################################################################################
    # Check object preservation when we're not supposed to have copying

    def checkUpdate_05_SameObjectPreserved_02_complex(self, overwrite, protect_structure):
        p = random_tree(0, 50)
        cp = p.copy()

        q = random_tree(0, 50)
        cq = q.copy()
        
        def check(r):
            for k, v in chain(cp.iteritems(), cq.iteritems()):
                self.assert_(r[k] is v)

        self.checkUpdate(p, q, False, check, overwrite, protect_structure)
 
    def testUpdate_05_SameObjectPreserved_02_complex_FF(self):
        self.checkUpdate_05_SameObjectPreserved_02_complex(False, False)

    def testUpdate_05_SameObjectPreserved_02_complex_FT(self):
        self.checkUpdate_05_SameObjectPreserved_02_complex(False, True)

    def testUpdate_05_SameObjectPreserved_02_complex_FF(self):
        self.checkUpdate_05_SameObjectPreserved_02_complex(True, False)

    def testUpdate_05_SameObjectPreserved_02_complex_FT(self):
        self.checkUpdate_05_SameObjectPreserved_02_complex(True, True)


    ################################################################################
    # Atomic stuff

    def testUpdate_06_atomic_01(self):

        p = TreeDict.fromkeys('abcdefghijklmnopqrstuvwxyz')
        q = TreeDict.fromkeys('abcdefghijklnopqrstuvwxyz')
        q.m.x = 1

        cp = p.copy()
        cq = q.copy()

        self.assertRaises(TypeError, lambda: p.update(q, protect_structure = True))

        self.assert_(cp == p)
        self.assert_(cq == q)

    ################################################################################
    # Proper attachments

    def checkUpdate_07_properAttaching(self, overwrite, protect_structure):
        p = random_tree(0, 50)
        cp = p.copy()

        q = random_tree(0, 50)
        cq = q.copy()
        
        def check(r):
            for k in chain(cp.iterkeys(recursive = True, branch_mode = 'only'),
                           cq.iterkeys(recursive = True, branch_mode = 'only')):
                
                self.assert_(r[k].rootNode() is r)

        self.checkUpdate(p, q, False, check, overwrite, protect_structure)

    def testUpdate_07_properAttaching_FF(self):
        self.checkUpdate_07_properAttaching(False, False)

    def testUpdate_07_properAttaching_FT(self):
        self.checkUpdate_07_properAttaching(False, True)

    def testUpdate_07_properAttaching_FF(self):
        self.checkUpdate_07_properAttaching(True, False)

    def testUpdate_07_properAttaching_FT(self):
        self.checkUpdate_07_properAttaching(True, True)


    ################################################################################
    # Whether Tree Values are properly handled

    def checkUpdate_08_TreeValuesProperlyHandled(self, deep_copy, overwrite, protect_structure):
        p = TreeDict()
        p.a = TreeDict(x = 1, y = 2)

        q = TreeDict()
        q.a.y = 3
        q.a.z = 4

        def check(r):
            self.assert_(r.a.x == 1)
            self.assert_(r.a.y == 3 if overwrite else 2)
            self.assert_(r.a.z == 4)

        self.checkUpdate(p, q, deep_copy, check, overwrite, protect_structure)

    def testUpdate_08_TreeValuesProperlyHandled_FFF(self):
        self.checkUpdate_08_TreeValuesProperlyHandled(False, False, False)

    def testUpdate_08_TreeValuesProperlyHandled_FFT(self):
        self.checkUpdate_08_TreeValuesProperlyHandled(False, False, True)

    def testUpdate_08_TreeValuesProperlyHandled_FTF(self):
        self.checkUpdate_08_TreeValuesProperlyHandled(False, True, False)

    def testUpdate_08_TreeValuesProperlyHandled_FTT(self):
        self.checkUpdate_08_TreeValuesProperlyHandled(False, True, True)
        
    def testUpdate_08_TreeValuesProperlyHandled_TFF(self):
        self.checkUpdate_08_TreeValuesProperlyHandled(True, False, False)

    def testUpdate_08_TreeValuesProperlyHandled_TFT(self):
        self.checkUpdate_08_TreeValuesProperlyHandled(True, False, True)

    def testUpdate_08_TreeValuesProperlyHandled_TTF(self):
        self.checkUpdate_08_TreeValuesProperlyHandled(True, True, False)

    def testUpdate_08_TreeValuesProperlyHandled_TTT(self):
        self.checkUpdate_08_TreeValuesProperlyHandled(True, True, True)

    def checkUpdate_09_ProperParents(self, deep_copy, overwrite, preserve_structure):

        p = TreeDict()

        ab = p.a.b = unique_object()
        ac = p.a.c = unique_object()
        b  = p.b   = unique_object()

        p2 = TreeDict()

        p2.update(p)

        self.assert_(p.a is not p2.a)
        self.assert_(p.a.rootNode() is p)
        self.assert_(p.a.parentNode() is p)
        self.assert_(p2.a.rootNode() is p2)
        self.assert_(p2.a.parentNode() is p2)

        self.assert_(p.a.b == p2.a.b)
        self.assert_(p.a.b is p2.a.b)
        
        self.assert_(p.a.c == p2.a.c)
        self.assert_(p.a.c is p2.a.c)
            
        self.assert_(p == p2)

    def testUpdate_10_rewritingLocal(self):

        # This test solidifies the following behavior

        p = TreeDict()
        p.a = TreeDict()

        q = TreeDict()
        q.a.x = 1

        p.update(q)

        self.assert_(p.a.isRoot())
        self.assert_(p.a.x == 1)

    def testUpdate_12_Atomic_01(self):

        p = TreeDict()
        p.makeBranch('b')
        p.b.freeze()

        p_before = p.copy()

        p2 = TreeDict(a = 1)
        p2["b.a"] = 4

        self.assert_(p_before == p)

        self.assertRaises(TypeError, lambda: p.update(p2))

        self.assert_(p_before == p)

    def testUpdate_12_Atomic_01b_large(self):

        p = random_tree()

        # Freeze b so it causes an error
        p.makeBranch('b')
        p.b.freeze()

        p_before = p.copy()

        p2 = TreeDict.fromkeys('acdefghijklmnopqrstuvwxyz')
        p2.update(random_tree())
        p2["b.a"] = 4

        self.assert_(p_before == p)

        self.assertRaises(TypeError, lambda: p.update(p2))

        self.assert_(p_before == p)

    def testUpdate_12_Atomic_02(self):

        p = TreeDict()
        p.a   = 1
        p.c   = 2

        p2 = TreeDict()
        p2.a.x = 3
        p2.c   = 4

        p_before = p.copy()

        self.assert_(p == p_before)

        self.checkUpdateBadStructure(p, p2)

        self.assert_(p == p_before)

    ################################################################################
    # Making sure branch values are not wrongly attached
 
    def checkUpdate_14_branchValuesNotAttached_01(self, overwrite, protect_structure):
        p = TreeDict()
        
        ax = p.a.x = TreeDict()
        ay = p.a.y = unique_object()
        p.b = 2

        p2 = TreeDict()
        az = p2.a.z = TreeDict()

        def check(r):
            self.assert_(r.a.x is ax)
            self.assert_(r.a.x.isRoot())
            self.assert_(r.a.y is ay)
            self.assert_(r.a.z is az)
            self.assert_(r.a.z.isRoot())
            self.assert_(r.b == 2)

        self.checkUpdate(p2, p, False, check, overwrite, protect_structure)

    def testUpdate_14_branchValuesNotAttached_01_FF(self):
        self.checkUpdate_14_branchValuesNotAttached_01(False, False)

    def testUpdate_14_branchValuesNotAttached_01_FT(self):
        self.checkUpdate_14_branchValuesNotAttached_01(False, True)

    def testUpdate_14_branchValuesNotAttached_01_TF(self):
        self.checkUpdate_14_branchValuesNotAttached_01(True, False)

    def testUpdate_14_branchValuesNotAttached_01_TT(self):
        self.checkUpdate_14_branchValuesNotAttached_01(True, True)

    ################################################################################
    # Dangling nodes handled properly

    def checkUpdate_15_DanglingNodesAttached(self, overwrite, protect_structure):
        p = TreeDict()
        a = p.a

        q = TreeDict()
        q.a.x = 1

        p.update(q, overwrite=overwrite, protect_structure =protect_structure)

        self.assert_(p.a.x == 1)
        self.assert_(a is p.a)
        self.assert_(a.rootNode() is p)

    def testUpdate_15_DanglingNodesAttached_FF(self):
        self.checkUpdate_15_DanglingNodesAttached(False, False)

    def testUpdate_15_DanglingNodesAttached_FT(self):
        self.checkUpdate_15_DanglingNodesAttached(False, True)

    def testUpdate_15_DanglingNodesAttached_TF(self):
        self.checkUpdate_15_DanglingNodesAttached(True, False)

    def testUpdate_15_DanglingNodesAttached_TT(self):
        self.checkUpdate_15_DanglingNodesAttached(True, True)

    ########################################

    def checkUpdate_16_DanglingNodesIgnored(self):
        p = TreeDict()
        p.a = 1

        q = TreeDict()
        q.a

        def check(r):
            self.assert_(r.a == 1)

        self.checkUpdate(p, q, False, check, overwrite=True, protect_structure=True)




if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python

# Copyright (c) 2009, Hoyt Koepke (hoytak@gmail.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     - Neither the name 'treedict' nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Hoyt Koepke ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Hoyt Koepke BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import random, unittest, cPickle, collections
from treedict import TreeDict, getTree
import treedict
from copy import deepcopy, copy

from hashlib import md5
import random

from common import *

class TestHashes(unittest.TestCase):

    ################################################################################
    # Hashes

    def testhashes_01(self):
        p1 = TreeDict('hashes_test_tree')
        p1.asdf = "1234"
        
        p2 = TreeDict('hashes_test_tree')
        p2.asdf = "1234"

        self.assert_(len(p1.hash()) != 0)
        self.assert_(p1.hash() == p2.hash())

    def testhashes_02(self):
        p1 = TreeDict('hashes_test_tree')
        p1.asdf = "1234"
        
        p2 = TreeDict('hashes_test_tree')
        p2.asdf = "1235"

        self.assert_(len(p1.hash()) != 0)
        self.assert_(p1.hash() != p2.hash())
        
    def testhashes_03(self):
        p1 = TreeDict('hashes_test_tree')
        p1.asdf = "1234"
        
        p2 = TreeDict('hashes_test_tree')
        p2.asdf = "1234"

        self.assert_(len(p1.hash("asdf")) != 0)
        self.assert_(p1.hash("asdf") == p2.hash("asdf"))


    def testhashes_04(self):
        p1 = TreeDict('hashes_test_tree')
        p1.asdf = 221.12345139
        
        p2 = TreeDict('hashes_test_tree')
        p2.asdf = 221.12345139

        self.assert_(len(p1.hash("asdf")) != 0)
        self.assert_(p1.hash("asdf") == p2.hash("asdf"))

    def testhashes_05(self):
        p1 = TreeDict('hashes_test_tree')
        p1.asdf = 1234498484884499393939999324322432
         
        p2 = TreeDict('hashes_test_tree')
        p2.asdf = 1234498484884499393939999324322432

        self.assert_(len(p1.hash("asdf")) != 0)
        self.assert_(p1.hash("asdf") == p2.hash("asdf"))

    def testhashes_06(self):
        p1 = TreeDict('hashes_test_tree')
        p1.asdf  = 1234498484884499393939999324322432
        
        p2 = TreeDict('hashes_test_tree')
        p2.a.b.c = 1234498484884499393939999324322432

        self.assert_(len(p1.hash("asdf")) != 0)
        self.assert_(p1.hash("asdf") != p2.hash("a"))
        self.assert_(p1.hash("asdf") != p2.hash("a.b"))

    def testhashes_07(self):
        p1 = TreeDict('hashes_test_tree')
        p1.asdf = 1234498484884499393939999324322432
        
        p2 = TreeDict('hashes_test_tree')
        p2.a.b.asdf = 1234498484884499393939999324322432

        self.assert_(len(p1.hash("asdf")) != 0)
        self.assert_(p1.hash("asdf") == p2.hash("a.b.asdf"))

    def testhashes_08(self):
        p1 = TreeDict('hashes_test_tree')
        p1.asdf = 1234498484884499393939999324322432
        
        p2 = TreeDict('hashes_test_tree')
        p2.a.b.notasdf = 1234498484884499393939999324322432

        self.assert_(len(p1.hash("asdf")) != 0)
        self.assert_(p1.hash("asdf") != p2.hash("a.b"))
        self.assert_(p1.hash() != p2.a.b.hash())
        self.assert_(p2.a.b.hash() == p2.hash("a.b"))

    def testhashes_09(self):
        p1 = TreeDict('hashes_test_tree')
        p1.a.b.c.asdf = 123
        
        p2 = TreeDict('hashes_test_tree')
        p2.a.b.asdf = 123

        self.assert_(len(p1.hash("a.b.c.asdf")) != 0)
        self.assert_(p1.hash("a.b.c") == p2.hash("a.b"))
        self.assert_(p1.hash("a.b.c.asdf") == p2.hash("a.b.asdf"))
        self.assert_(p1.a.b.c.hash() == p1.hash("a.b.c"))
        self.assert_(p2.a.b.hash() == p2.hash("a.b"))

    def testhashes_10(self):
        p1 = TreeDict('hashes_test_tree')
        p1.a.b.c.asdf = deepcopy(test_object)
        
        p2 = TreeDict('hashes_test_tree')
        p2.a.b.c.asdf = deepcopy(test_object)

        self.assert_(p1.hash() == p2.hash())

    def testhashes_11(self):
        p1 = TreeDict('hashes_test_tree')
        p1.a.b.c.asdf = deepcopy(test_tuple)
        
        p2 = TreeDict('hashes_test_tree')
        p2.a.b.c.asdf = deepcopy(test_tuple)

        self.assert_(p1.hash() == p2.hash())

    def testhashes_12(self):
        p1 = TreeDict('hashes_test_tree')
        p1.a.b.c.asdf = deepcopy(test_list)
        
        p2 = TreeDict('hashes_test_tree')
        p2.a.b.c.asdf = deepcopy(test_list)
        
        self.assert_(p1.hash() == p2.hash())

    def testhashes_13_frozen_consistency(self):

        p1 = sample_tree()
        h = p1.hash()

        p1.freeze()

        self.assert_(p1.hash() == h)

    def testhashes_14_frozen_consistency(self):
        p1 = sample_tree()
        p1.z.a.b.c = 123

        h = p1.hash()
        p1.z.a.freeze()

        self.assert_(p1.hash() == h)

    def testhashes_15_attaching(self):
        p1 = TreeDict('root')
        p1.b1 = TreeDict('b1')

        h = p1.hash()

        p1.attach(copy = False, recursive = True)
        
        self.assert_(p1.hash() != h)

    def testhashes_15_attaching(self):
        p1 = TreeDict('root')
        p1.b1 = TreeDict('b1')
        p1.b1.b2 = TreeDict('b2')

        h = p1.hash()

        p1.b1.attach(copy = False, recursive = True)
        
        self.assert_(p1.hash() != h)
        
    def testhashes_16_pythonkeys(self):
        p1 = TreeDict()
        p1.a.b = 123

        def f(): return {p1 : 0}

        self.assertRaises(TypeError, f)

        p1.freeze()

        # Should work now
        d = {p1 : 0}


    def testhashes_17_pythonkeys(self):
        p1 = TreeDict()
        p1.a.b = [123, 43]
        p1.freeze()

        def f(): return {p1 : 0}

        self.assertRaises(TypeError, f)

    def testhashes_18_EqualityTesting_01(self):
        p = sample_tree()
        p1 = p.copy(deep=True)
        p2 = p.copy(deep=True)

        self.assert_(p1 == p2)
        self.assert_(p1.hash() == p2.hash())

        p1.mydict = {1 : 2}
        p2.mydict = {1 : 3}

        self.assert_(p1 != p2)
        self.assert_(p1.hash() != p2.hash())

    def testhashes_19_Keys_01_reversing(self):
        p = TreeDict(a = 1, b = 2, c = 3)
        self.assert_(p.hash(keys=['a', 'b']) == p.hash(keys=['b', 'a']))

    def testhashes_19_Keys_02_full(self):
        p = TreeDict(a = 1, b = 2, c = 3)
        self.assert_(p.hash(keys=['a', 'b', 'c']) == p.hash())

    def testhashes_19_Keys_03_single(self):
        p = TreeDict(a = 1, b = 2, c = 3)
        self.assert_(p.hash(keys=['a']) == p.hash('a'))

    def testhashes_19_Keys_04_nonintersection(self):
        p = TreeDict(a = 1, b = 2, c = 3)
        self.assert_(p.hash(keys=['a', 'b']) != p.hash(keys=['a','c']))

    def testhashes_20a_infinite_recursion_raises_error(self):
        p = TreeDict()

        p.makeBranch("a")

        p.a.b.c = p.a

        self.assertRaises(RuntimeError, lambda: p.hash())
        
    def testhashes_20b_infinite_recursion_raises_error__frozen(self):
        p = TreeDict()

        p.makeBranch("a")

        p.a.b.c = p.a

        p.freeze()

        self.assertRaises(RuntimeError, lambda: p.hash())
        


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python

# Copyright (c) 2009, Hoyt Koepke (hoytak@gmail.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     - Neither the name 'treedict' nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Hoyt Koepke ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Hoyt Koepke BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import random, unittest, cPickle, collections
from treedict import TreeDict, getTree
import treedict
from copy import deepcopy, copy

from hashlib import md5
import random

from common import *

class TestCopying(unittest.TestCase):

    ############################################################
    # Copying the tree

    def testCopying_01(self):
        p1 = TreeDict('root')
        p1.a = 123

        p2 = copy(p1)

        self.assert_(p1 == p2)
        self.assert_(p1.hash() == p2.hash())
        
    def testCopying_01d(self):
        p1 = TreeDict('root')
        p1.a = 123

        p2 = deepcopy(p1)

        self.assert_(p1 == p2)
        self.assert_(p1.hash() == p2.hash())

    def testCopying_02_dangling(self):
        p1 = TreeDict('root')
        p1.a

        p2 = copy(p1)

        self.assert_(p1 == p2)
        self.assert_(p1.hash() == p2.hash())

    def testCopying_02d_dangling(self):
        p1 = TreeDict('root')
        p1.a

        p2 = deepcopy(p1)

        self.assert_(p1 == p2)
        self.assert_(p1.hash() == p2.hash())

    def testCopying_03_Dangling_AttributeError(self):
        
        p = TreeDict()

        def f():
            p.a.b.copy()

        self.assertRaises(AttributeError, f)

    def testCopying_04_dangling_AttributeError_correct_node(self):
        
        p = TreeDict()
        ae_msg = ''

        try:
            p.aduvulksjucmfiddkjdo.b.copy()
        except AttributeError, ae:
            ae_msg = str(ae)

        # Make sure it propegates up so the error message is sent from
        # the root dangling node.
        self.assert_('aduvulksjucmfiddkjdo' in ae_msg)


    def testCopying_05(self):
        
        p = TreeDict('root')
        b = p.makeBranch('b').copy()

        self.assert_(p.b.branchName() == 'b', p.b.branchName())
        self.assert_(b.branchName(add_tree_name = True) == 'b', b.branchName(add_tree_name = True))
        self.assert_(p.b.branchName(add_path = True, add_tree_name = True) == 'root.b',
                     p.b.branchName(add_path = True, add_tree_name = True))

    def testCopying_05b(self):
        
        p = TreeDict('root')
        b = copy(p.makeBranch('b'))

        self.assert_(p.b.branchName() == 'b', p.b.branchName())
        self.assert_(b.branchName(add_tree_name = True) == 'b', b.branchName(add_tree_name = True))
        self.assert_(p.b.branchName(add_path = True, add_tree_name = True) == 'root.b',
                     p.b.branchName(add_path = True, add_tree_name = True))

    def testCopying_05c(self):
        
        p = TreeDict('root')
        b = deepcopy(p.makeBranch('b'))

        self.assert_(p.b.branchName() == 'b', p.b.branchName())
        self.assert_(b.branchName(add_tree_name = True) == 'b', b.branchName(add_tree_name = True))
        self.assert_(p.b.branchName(add_path = True, add_tree_name = True) == 'root.b',
                     p.b.branchName(add_path = True, add_tree_name = True))

    def testCopying_05b_central_system(self):
        # This test constrains the central system to have the same
        # name system as regular trees (I was tempted to do it
        # otherwise to make the constraints of copied branches
        # easier.)

        p = getTree('tc_root')
        b = p.makeBranch('b').copy()
    
        self.assert_(b.branchName(add_tree_name = True) == 'b', b.branchName(add_tree_name = True))
        self.assert_(p.b.branchName(add_path = True, add_tree_name = True) == 'tc_root.b',
                     p.b.branchName(add_path = True, add_tree_name = True))

    def testCopying_05b_central_system(self):
        # This test constrains the central system to have the same
        # name system as regular trees (I was tempted to do it
        # otherwise to make the constraints of copied branches
        # easier.)

        p = getTree('tc_root')
        b = p.makeBranch('b').copy()
    
        self.assert_(b.branchName(add_tree_name = True) == 'b', b.branchName(add_tree_name = True))
        self.assert_(p.b.branchName(add_path = True, add_tree_name = True) == 'tc_root.b',
                     p.b.branchName(add_path = True, add_tree_name = True))

    def testCopying_05b_central_system(self):
        # This test constrains the central system to have the same
        # name system as regular trees (I was tempted to do it
        # otherwise to make the constraints of copied branches
        # easier.)

        p = getTree('tc_root')
        b = p.makeBranch('b').copy()
    
        self.assert_(b.branchName(add_tree_name = True) == 'b', b.branchName(add_tree_name = True))
        self.assert_(p.b.branchName(add_path = True, add_tree_name = True) == 'tc_root.b',
                     p.b.branchName(add_path = True, add_tree_name = True))

    def testCopying_06_setting_linked_branch_02_copy(self):
        p = TreeDict()

        p.a = p.defs.a1

        p.defs.a1.v = 1

        p2 = p.copy()

        p2.defs.a1.v = 2

        self.assert_(p.a.v == 1)
        self.assert_(p2.a.v == 2)

    def testCopying_06_setting_linked_branch_04_deepcopy_func(self):
        p = TreeDict()

        p.a = p.defs.a1

        p.defs.a1.v = 1

        p2 = deepcopy(p)  # only difference from above

        p2.defs.a1.v = 2

        self.assert_(p.a.v == 1)
        self.assert_(p2.a.v == 2)

    def testCopying_07_unfrozen(self):
        p = TreeDict()

        p.x = 1
        p.b.c.x = 2
        p.b.c.y = 1

        p.freeze()

        self.assert_(p.isFrozen())
        self.assert_(p.b.isFrozen())
        self.assert_(p.b.c.isFrozen())

        p2 = p.copy()

        self.assert_(not p2.isFrozen())
        self.assert_(not p2.b.isFrozen())
        self.assert_(not p2.b.c.isFrozen())
        
        p3 = copy(p)
        
        self.assert_(not p3.isFrozen())
        self.assert_(not p3.b.isFrozen())
        self.assert_(not p3.b.c.isFrozen())

    def testCopying_07a_Regression(self):

        p = TreeDict()

        p.a.b = [1,2,3,4,534]
        p.a.c = [1,22,534]
        p.b   = [1,2,3]

        p2 = TreeDict()

        p2.a.b = [1,2,3,4,534]
        p2.a.c = [1,22,534]
        p2.b   = [1,2,3]

        p3 = copy(p)
        
        self.assert_(p == p2)
        self.assert_(p == p3)

    def testCopying_07a_Regression_mutability_count_test(self):
        
        p = TreeDict()

        p.a.b = [1,2,3,4,534]
        p.a.c = [1,22,534]
        p.b   = [1,2,3]

        p2 = copy(p)

        self.assert_(p._numMutable() == p2._numMutable())
        self.assert_(p.a._numMutable() == p2.a._numMutable())

    def testCopying_07b_Regression_empty_branches(self):

        p = TreeDict()
        p.makeBranch("a")

        p2 = copy(p)
       
        self.assert_(p.a == p2.a)
        self.assert_(p.a.parentNode() is p)
        self.assert_(p2.a.parentNode() is p2)
        self.assert_(p.a is not p2.a)

    def testCopying_08_TreeDictValues(self):

        p = TreeDict()

        ab = p.a.b = TreeDict()

        q = p.copy()

        self.assert_(p == q)
        self.assert_(p.a is not q.a)
        self.assert_(ab is q.a.b)

    def testCopying_09_LinkedValuesPreserved(self):

        p = TreeDict()
        p.a.x = 1
        p.al = p.a

        q = p.copy()

        self.assert_(p.a is not q.a)
        self.assert_(q.al is q.a)
        
    def testCopying_09_LinkedValuesPreserved(self):

        p = TreeDict()
        p.a.x = 1
        p.b = p.a

        q = p.copy()

        self.assert_(p.a is not q.a)
        self.assert_(q.b is q.a)

    def testCopying_09_LinkedValuesPreserved_Reversed(self):

        p = TreeDict()
        p.b.x = 1
        p.a = p.b

        q = p.copy()

        self.assert_(p.b is not q.b)
        self.assert_(q.b is q.a)

    def testCopying_09_LinkedValuesPreserved(self):

        p = TreeDict()
        p.a.x = 1
        p.b = p.a

        q = p.copy()

        self.assert_(p.a is not q.a)
        self.assert_(q.b is q.a)

    def testCopying_10_Cousins(self):

        p = TreeDict()
        p.a.c.x = 1
        p.b.c = p.a.c

        q = p.copy()

        self.assert_(p.a is not q.a)
        self.assert_(p.a.c is not q.a.c)
        self.assert_(q.b.c is q.a.c)

    def testCopying_10_Cousins_subcopy(self):

        p = TreeDict()
        p.a.c.x = 1
        p.b.c = p.a.c
        p.b.d.x = 1

        qb = p.b.copy()

        self.assert_(qb.c.rootNode() is p)
        self.assert_(qb.d.rootNode() is qb)
        self.assert_(qb.d.x == 1)

    def testCopying_11_Cousins_multiple_copy(self):
        p = random_selflinked_tree(0, 4)
        # print "\np_before = "
        # print p.makeReport()

        q = p.copy()

        # print "\np = "
        # print p.makeReport()

        # print "\nq = "
        # print q.makeReport()

        self.assert_(q == p)

    def testCopying_12_Large_straight(self):
        p = random_selflinked_tree(0, 100)

        q = p.copy()

        for k, v in p.iteritems(branch_mode='all', recursive = True):

            if type(v) is TreeDict:

                pbn = p[k].branchName(add_path = True)
                qbn = q[k].branchName(add_path = True)

                self.assert_(pbn == qbn, "qbn = %s != %s = pbn" % (qbn, pbn))

                self.assert_(p[k] is not q[k])
                self.assert_(p[k] == q[k])

    def testCopying_13_Large_interwoven(self):
        p = random_selflinked_tree(0, 100)

        for bk, b in p.iteritems(recursive = False, branch_mode = 'only'):

            q = b.copy()

            # print "#"*40

            # print "\nbk = %s; b = " % bk
            # print b.makeReport()

            # print "\nbk = %s q = " % bk
            # print q.makeReport()

            for k, v in b.iteritems(branch_mode='all', recursive = True):

                if type(v) is TreeDict:

                    pbn = v.branchName(add_path = True)
                    qbn = q[k].branchName(add_path = True)

                    if q[k].rootNode() is not q:
                        self.assert_(q[k].rootNode() is p)
                        self.assert_(q[k] is v)
                    else:
                        self.assert_(v is not q[k])
                        self.assert_(pbn == bk + '.' + qbn, "pbn='%s' != '%s' = bk+qbn" % (pbn, bk+'.' + qbn))

    # Also need tests covering cases where flags are cleared on copied
    # nodes

    # Also need to add tests for copying with links and that the
    # proper link-parental relations are preserved in copying.

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python

# Copyright (c) 2009, Hoyt Koepke (hoytak@gmail.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     - Neither the name 'treedict' nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Hoyt Koepke ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Hoyt Koepke BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import random, unittest, cPickle, collections
from treedict import TreeDict, getTree
import treedict
from copy import deepcopy, copy

from hashlib import md5
import random

from common import *

class TestPruning(unittest.TestCase):

    def testpruning_01(self):
        p = TreeDict('prunetest')
        p.a.b.c = 123
        h1 = p.hash()

        self.assert_('a.b.c' in p)
        
        p.pop('a.b.c')

        self.assert_('a.b.c' not in p)

        h2 = p.hash()

        p.a.b.c = 123
        h3 = p.hash()

        self.assert_(h1 != h2)
        self.assert_(h1 == h3)

    def testpruning_02(self):
        p = TreeDict('prunetest')
        p.a.b.c = 123
        h1 = p.hash()

        self.assert_('a.b.c' in p)
        
        p.pop('a.b.c', prune_empty=True)

        self.assert_('a.b' not in p)

        h2 = p.hash()

        p.a.b.c = 123
        h3 = p.hash()

        self.assert_(h1 != h2)
        self.assert_(h1 == h3)

    def testpruning_03(self):
        p = TreeDict('prunetest')
        p.a.b.c = 123
        h1 = p.hash()

        self.assert_('a.b.c' in p)

        del p.a.b.c

        self.assert_('a.b.c' not in p)

        h2 = p.hash()

        p.a.b.c = 123
        h3 = p.hash()

        self.assert_(h1 != h2)
        self.assert_(h1 == h3)

    def testpruning_04(self):
        p1 = sample_tree()
        p1.testnode = 123

        p2 = deepcopy(p1)

        self.assert_(p1 == p2)

        p1.pop("testnode")

        self.assert_(p1 != p2)
        self.assert_('testnode' not in p1)

    def testpruning_05_silent(self):
        p1 = sample_tree()
        p1.testnode = 123

        p2 = deepcopy(p1)

        self.assert_(p1 == p2)

        self.assertRaises(KeyError, lambda: p1.pop("testnode_nonexistent"))
                          
        p1.pop("testnode_nonexistent", silent=True)

        self.assert_(p1 == p2)
        self.assert_('testnode' in p1)

        p1.pop("testnode", silent=True)

        self.assert_(p1 != p2)
        self.assert_('testnode' not in p1)

    def testpruning_06_silent_popitem(self):
        p1 = sample_tree()
        p1.testnode = 123

        p2 = deepcopy(p1)

        self.assert_(p1 == p2)

        self.assertRaises(KeyError, lambda: p1.popitem("testnode_nonexistent"))
                          
        p1.pop("testnode_nonexistent", silent=True)

        self.assert_(p1 == p2)
        self.assert_('testnode' in p1)

        p1.popitem("testnode", silent=True)

        self.assert_(p1 != p2)
        self.assert_('testnode' not in p1)

class TestDeletion(unittest.TestCase):

    def testDeletion_01(self):
        
        p = TreeDict()
        p.a = 1

        self.assert_('a' in p)

        del p['a']

        self.assert_('a' not in p)


    def testDeletion_02(self):
        
        p = TreeDict()
        p.a.b.c = 1

        self.assert_('a.b.c' in p)

        del p['a.b.c']

        self.assert_('a' in p)
        self.assert_('a.b' in p)
        self.assert_('a.b.c' not in p)

    def testDeletion_03_is_detached(self):
        p = TreeDict()
        p.makeBranch("a")
        a = p.a

        del p["a"]

        self.assert_(a.rootNode() is not p)
        self.assert_(a.parentNode() is None)
        
    def testDeletion_04_properException_Attribute(self):
        p = TreeDict()

        def f():
            del p.a
        
        self.assertRaises(AttributeError, f)

    def testDeletion_05_properException_Keys_1(self):
        p = TreeDict()

        def f():
            del p["a"]
        
        self.assertRaises(KeyError, f)

    def testDeletion_05_properException_Keys_2(self):
        p = TreeDict()
        def f():
            del p[0]
        
        self.assertRaises(KeyError, f)

    def testDeletion_05_properException_Keys_3(self):
        p = TreeDict()
        def f():
            del p[None]
        
        self.assertRaises(KeyError, f)

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python

# Copyright (c) 2009, Hoyt Koepke (hoytak@gmail.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     - Neither the name 'treedict' nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Hoyt Koepke ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Hoyt Koepke BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import random, unittest, cPickle, collections
from treedict import TreeDict, getTree
import treedict
from copy import deepcopy, copy

from hashlib import md5
import random

from common import *

class TestSetting(unittest.TestCase):

    def test_Frozen(self):
        p = sample_tree()
        p.freeze()
        self.assertRaises(TypeError, lambda: p.set("asdf", 0))

    def testSetGet01(self):
        p = sample_tree()
        
        p.set("aabsd", 123)

        self.assert_(p.aabsd == 123)

    def testSetGet02(self):
        p = sample_tree()
        p.a = 123

        self.assert_(p.get("a") == 123)

    def testSetGet03(self):
        p = TreeDict()
        p.a.b.c.d.e.f = 1
        self.assert_(p.get("a.b.c.d.e.f") == 1)

    def testSetGet04(self):
        p = sample_tree()
        p.set("b.c", 123)

        self.assert_(p.get("b.c") == 123)

    def testSetGet04b(self):
        p = sample_tree()
        p.makeBranch("b")
        p.set("b.c", 123)

        self.assert_(p.get("b.c") == 123)

    def testSetGet05(self):
        p = getTree('a_s3kkdf93kd9k3ikdkmd') # unique
        p.a.b.c.d.e.f = 1
        self.assert_(p.get("a.b.c.d.e.f") == 1)

    def testSetGet06(self):

        # regression test
        p = getTree('upt_08_bcg1_')
        p.a18462.a1732643.x = 1
        q = p.a18462.copy()
        q.a1732643.a1232 = "borkbork"  # should be bad
        
        self.assert_(p.a18462.a1732643.x == 1)
        self.assert_(q.a1732643.a1232 == "borkbork")
        self.assert_(q.a1732643.x == 1)
        
    def testSetGet07(self):

        # Regression
        p = getTree('upt_08_bcg2_whodunit_')
        self.assert_(not p.isFrozen())
        p.a18462.a1732643.x = 1
        q = p.a18462.a1732643.copy()
        q.a1232 = "borkbork"  

        self.assert_(p.a18462.a1732643.x == 1)
        self.assert_(q.a1232 == "borkbork")
        self.assert_(q.x == 1)


    ############################################################
    # Testing more advanced functionality of the set() method

    def testSet_01(self):
        p = sample_tree()
        v1 = (1,2,3)

        p.set("a.b", v1) 
        self.assert_(p.a.b is v1)

    def testSet_02(self):
        p = sample_tree()
        v1 = (1,2,3)
        v2 = (3,2,1)

        p.set("a.b", v1, "a.c", v2)
        self.assert_(p.a.b is v1)
        self.assert_(p.a.c is v2)

    def testSet_03_SetOrder(self):
        p = sample_tree()
        v1 = (1,2,3)
        v2 = (3,2,1)

        p.set("a.b", v1, "a.b", v2)
        self.assert_(p.a.b is v2)

    def testSet_04_mixKWargs(self):
        p = sample_tree()
        v1 = (1,2,3)
        v2 = (3,2,1)

        p.set("a.b", v1, **{"b" : v2})

        self.assert_(p.a.b is v1)
        self.assert_(p.b is v2)
        
    def testSet_05_kwargs(self):
        p = sample_tree()
        v1 = (1,2,3)
        v2 = (3,2,1)

        p.set(**{"a" : v1, "b" : v2})

        self.assert_(p.a is v1)
        self.assert_(p.b is v2)
            
    def testSet_05b_kwargs(self):
        p = sample_tree()
        v1 = (1,2,3)
        v2 = (3,2,1)

        p.set(a = v1, b = v2)

        self.assert_(p.a is v1)
        self.assert_(p.b is v2)

    def testSet_06_kwargs_large_dict(self):
        d = {}

        for i in range(10000):
            d["a%d" % i] = i

        p = sample_tree()

        p.set(**d)

        for i in range(10000):
            self.assert_(p["a%d" % i] == i)

    def testSet_07_kwargs_precedence(self):
        p = sample_tree()
        v1 = (1,2,3)
        v2 = (3,2,1)

        p.set("a", v1, a = v2)

        # Keyword arguments
        self.assert_(p.a is v2)

    def testSet_08_validation_first(self):
        p = sample_tree()
        v1 = (1,2,3)
        
        try:
            p.set("a", v1, "0123", v2)
        except Exception:
            pass

        self.assert_("a" not in p)

    def testSet_09_validation_first(self):
        p = sample_tree()
        v1 = (1,2,3)
        v2 = (3,2,1)
        
        p.a = v1

        try:
            p.set("a", v2, "0123", None)
        except Exception:
            pass

        self.assert_(p.a is v1)

    def testSet_10_validation_first(self):
        p = TreeDict()
        v1 = (1,2,3)
        v2 = (3,2,1)
        
        p.a = v1

        p1 = deepcopy(p)

        try:
            p.set("a", v2, "0123", None)
        except Exception:
            pass

        self.assert_(p.a is v1)
        self.assert_(p1 == p)

    def testSet_11_validation_first(self):
        d = {}

        tn = unique_name()

        for i in range(10000):
            d["a%d" % i] = i

        p = treedict.getTree(tn)
        
        p1 = deepcopy(p)

        self.assert_(p1 == p)
        
    def testSet_12_bad_args(self):
        p = sample_tree()
        self.assertRaises(TypeError, lambda: p.set('a', 1, None, 1))

    def testSet_13_bad_args(self):
        p = sample_tree()
        self.assertRaises(TypeError, lambda: p.set('a', 1, 123, 1))

    def testSet_14_bad_args(self):
        p = sample_tree()
        self.assertRaises(NameError, lambda: p.set('a', 1, '', 1))
 
    def testSet_15_bad_args(self):
        p = sample_tree()
        self.assertRaises(NameError, lambda: p.set('a', 1, '123', 1))

    def testSet_16_bad_args(self):
        p = sample_tree()
        self.assertRaises(TypeError, lambda: p.set('a', 1, 'abc'))

    def testSet_17_bad_args(self):
        p = sample_tree()
        self.assertRaises(TypeError, lambda: p.set('a'))

    def testSet_19_overwriting_value_with_branch_01_okay(self):
        p = TreeDict()

        p.a = 12

        p.set("a.b.c", 1, protect_values=False)
        
        self.assert_(p.a.b.c == 1)

    def testSet_19_overwriting_value_with_branch_02_nochange_on_bad(self):
        p = TreeDict()

        p.a = 12

        pc = p.copy()

        self.assertRaises(NameError, lambda: p.set("a.b.c.123.d", 1, protect_structure = False))

        self.assert_(p.a == 12)
        self.assert_(p == pc)

    def testSet_19_overwriting_value_with_branch_03_error(self):
        p = TreeDict()

        p.a = 12

        self.assertRaises(TypeError, lambda: p.set("a.b.c", 1, protect_structure = True))

    def testSet_20_setting_linked_branch_01(self):
        p = TreeDict()

        p.a = p.defs.a1

        p.defs.a1.v = 1

        self.assert_(p.a.v == 1)

    def testSet_20_setting_linked_branch_02(self):
        p = TreeDict()

        p.a = p.defs.a1

        p.set("defs.a1.v", 1)

        self.assert_(p.a.v == 1)

    def testSet_21_Setting_ThroughTreeDict_01(self):
        p = TreeDict()

        p.a.b = TreeDict()

        p.set("a.b.c.d.x", 1)

        self.assert_(p.a.b.c.d.x == 1)
        self.assert_(p.a.b.isRoot())

    def testSet_21_Setting_ThroughTreeDict_02(self):
        p = TreeDict()
        p.a.b = TreeDict()
        p["a.b.c.d.x"] = 1

        self.assert_(p.a.b.c.d.x == 1)
        self.assert_(p.a.b.isRoot())
        
    def testSetInit_01(self):
        p = TreeDict(a = 1, b = 2)
        self.assert_(p.a == 1)
        self.assert_(p.b == 2)

    ############################################################
    # Testing the checkset function

    def testDrySet_01(self):
        p = sample_tree()
        p1 = p.copy(deep = True)
        p2 = p.copy(deep = True)

        p1.checkset(a = 123)

        self.assert_(p1 == p2)

    def testDrySet_02(self):
        p1 = sample_tree().copy(deep=True)

        p1.v = 100
        p1.a.b.c = 100

        p2 = deepcopy(p1)

        p1.checkset(v = 123)

        self.assert_(p1.v != 123)
        self.assert_(p1.v == 100)
        self.assert_(p1 == p2)
        
    def testDrySet_03(self):
        p = sample_tree()
        
        p1 = p.copy(deep=True)
        p2 = p.copy(deep=True)

        p1.checkset("a.b.c.d", 123)

        self.assert_(p1 == p2)

    def testDrySet_04_value_overwrite(self):
        p = TreeDict()
        p.b = 1

        p_before = p.copy()

        self.assertRaises(TypeError, lambda: p.checkset("b.a", None, protect_structure = True))

        self.assert_(p_before == p)

    def testDrySet_05_makeBranch_Frozen(self):
        p = TreeDict()
        p.makeBranch('b')
        p.b.freeze()

        p_before = p.copy()

        self.assertRaises(TypeError, lambda: p.checkset("b.a", None))

        self.assert_(p_before == p)

    ############################################################
    # Setting arguments from string

    def testSetFromString_01(self):
        p = sample_tree()
        
        self.assert_(p.setFromString("a", "(0,1,2)"))
        self.assert_(p.a == (0,1,2))

    def testSetFromString_02(self):
        p = sample_tree()
        
        self.assert_(p.setFromString("a", "'string'"))
        self.assert_(p.a == 'string')

    def testSetFromString_03(self):
        p = sample_tree()
        
        self.assert_(p.setFromString("a", "123 + 345"))
        self.assert_(p.a == 123 + 345)

    def testSetFromString_04(self):
        p = sample_tree()
        
        self.assert_(not p.setFromString("a", "123 345"))
        self.assert_(p.a == "123 345")
        
    def testSetFromString_05(self):
        p = sample_tree()

        self.assert_(p.setFromString("a", "x/2", {"x" : 4}))
        self.assert_(p.a == 2)


    ################################################################################
    # Testing the dict interface

    def testSet_nochange_on_failure(self):
        p = sample_tree()

        p1 = p.copy(deep=True)
        p2 = p.copy(deep=True)
        
        def f():
            p1.set("a.b.c.d.123", 1)

        self.assertRaises(NameError, f)

        self.assert_(p1 == p2)

    def testDictSet_nochange_on_failure(self):
        p = sample_tree()

        p1 = p.copy(deep=True)
        p2 = p.copy(deep=True)
        
        def f():
            p1["a.b.c.d.123"] = 1

        self.assertRaises(NameError, f)

        self.assert_(p1 == p2)

    def testFromKeys_01_setting(self):

        key_iterable = ["a", "b", "cdef"]

        p = TreeDict.fromkeys(key_iterable)

        for n in key_iterable:
            self.assert_(p.get(n) is None)


    def testFromKeys_02_setting(self):
        v = ["asdf"]

        key_iterable = ["a", "b", "cdef"]

        p = TreeDict.fromkeys(key_iterable, v)

        for n in key_iterable:
            self.assert_(p.get(n) is v)


    def testFromKeys_03_checking_01(self):

        key_iterable = ["alphabet", 123]

        def f():
            TreeDict.fromkeys(key_iterable)
        
        self.assertRaises(TypeError, f)

    def testFromKeys_03_checking_02(self):

        key_iterable = "alphabet1"

        def f():
            p = TreeDict.fromkeys(key_iterable)
                    
        self.assertRaises(NameError, f)
        
    def testFromKeys_04_nochangetostring(self):
        
        p = sample_tree()
        key_iterable = ["abcdefg"]

        p.fromkeys(key_iterable)
        
        self.assert_(p == sample_tree())

    # Add in fromdict testing here

    def testFromDict_01(self):
        pass

    def testOrdering_01(self):
        t = TreeDict()

        t.a = 1
        t.b = 2

        self.assert_(t._getSettingOrderPosition('a') == (0,), t._getSettingOrderPosition('a'))
        self.assert_(t._getSettingOrderPosition('b') == (1,), t._getSettingOrderPosition('b'))

    def testOrdering_02(self):

        t = TreeDict()

        t.a   = 1
        t.b.c = 2
        t.x.y = 3
        t.b.a = 4
        
        self.assert_(t._getSettingOrderPosition('a') == (0,),  t._getSettingOrderPosition('a'))
        self.assert_(t._getSettingOrderPosition('b') == (1,), t._getSettingOrderPosition('b'))
        self.assert_(t._getSettingOrderPosition('x') == (2,), t._getSettingOrderPosition('x'))
        self.assert_(t._getSettingOrderPosition('b.c') == (1,0), t._getSettingOrderPosition('b.c'))
        self.assert_(t._getSettingOrderPosition('b.a') == (1,1), t._getSettingOrderPosition('b.a'))
        self.assert_(t._getSettingOrderPosition('x.y') == (2,0), t._getSettingOrderPosition('x.y'))

if __name__ == '__main__':
    unittest.main()

